package com.msdw.tms.api;

import com.msdw.tms.common.utils.R;
import com.msdw.tms.entity.ExperimentalReportEntity;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.RequestParam;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

@Api(value = "成绩管理", tags = "成绩管理")
public interface AchievementManagementApi {

    @ApiOperation(value = "查询指导老师", notes = "查询指导老师")
    R getTeacher(Integer projectId);

    @ApiOperation(value = "查询用户实验报告", notes = "查询用户实验报告")
    R getReport(Integer userId, Integer reportId);

    @ApiOperation(value = "成绩管理列表", notes = "查询成绩管理列表")
    R list(@RequestParam(value = "系统id", required = true) Integer systemId,
           @RequestParam(value = "分类", required = false) Integer projectPermissions,
           @RequestParam(value = "搜索内容", required = false) String searchContant,
           @RequestParam(value = "开始时间", required = false) String startTime,
           @RequestParam(value = "结束时间", required = false) String endTime,
           @RequestParam(value = "月份", required = false) Integer month,
           @RequestParam(value = "页码", required = true) Integer page,
           @RequestParam(value = "页数大小", required = true) Integer size);

    @ApiOperation(value = "查询实验教学列表", notes = "查询实验教学列表")
    R newList(@RequestParam(value = "用户id", required = true) Integer userId,
              @RequestParam(value = "搜索内容", required = false) String searchContant,
              @RequestParam(value = "开始时间", required = false) String startTime,
              @RequestParam(value = "结束时间", required = false) String endTime,
              @RequestParam(value = "月份", required = false) Integer month,
              @RequestParam(value = "系统",required = true) Integer systemId,
              @RequestParam(value = "页码", required = true) Integer page,
              @RequestParam(value = "页数大小", required = true) Integer size);

    @ApiOperation(value = "成绩管理列表", notes = "查询成绩管理列表")
    R reportList(@RequestParam(value = "系统id", required = true) Integer systemId,
                 @RequestParam(value = "项目id", required = true) Integer projectId,
                 @RequestParam(value = "搜索内容", required = false) String searchContant,
                 @RequestParam(value = "页码", required = true) Integer page,
                 @RequestParam(value = "页数大小", required = true) Integer size);

    @ApiOperation(value = "川大新增实验记录", notes = "川大新增实验记录")
    R addReport(ExperimentalReportEntity entity);

    @ApiOperation(value = "查询实验记录", notes = "查询实验记录")
    R queryReport(@ApiParam(value = "实验报告主键id", required = true) Integer reportId);

//    @ApiOperation(value = "直接修改实验记录分数",notes = "直接修改实验记录分数")
//    R updateScore(@ApiParam(value = "recordid和score") ProjectRecordEntity entity);

    @ApiOperation(value = "删除实验报告", notes = "根据实验记录主键reportId和实验报告主键recordId删除实验报告")
    R deleteReport(@ApiParam(value = "实验记录主键", required = true) List<Integer> reportId, @ApiParam(value = "实验报告主键", required = true) @RequestParam List<Integer> recordId);

    @ApiOperation(value = "班级实验报告列表查询", notes = "班级实验报告列表查询")
    R queryEvaluationReport(@ApiParam(value = "搜索内容（学校名称/学生姓名）", required = false) String searchContant,
                            @ApiParam(value = "项目", required = true) Integer projectId,
                            @ApiParam(value = "当前页码", required = true) Integer page,
                            @ApiParam(value = "每页显示数据量", required = true) Integer size);

    @ApiOperation(value = "班级实验成绩导出", notes = "班级实验成绩导出")
    void exportAchievement(HttpServletResponse response, @ApiParam(value = "实验记录集合recordId(此处为ids字符串（1，2，3）)") String ids, @ApiParam(value = "来源(1.练习2.考核)") Integer source) throws Exception;

    @ApiOperation(value = "成绩管理列表", notes = "成绩管理列表筛选查询")
    R queryAchievement(@ApiParam(value = "平台id", required = false) Integer systemId,
                       @ApiParam(value = "项目类型（0练习，1考核）", required = false) Integer projectPermissions,
                       @ApiParam(value = "搜索内容（项目名称/实验班级）", required = false) String searchContant,
                       @ApiParam(value = "开始时间", required = false) String startingtime,
                       @ApiParam(value = "结束时间", required = false) String endtime,
                       @ApiParam(value = "最近几月", required = false) Integer month);

//    @ApiOperation(value = "修改教师评语",notes = "修改教师评语:评语comment和实验报告主键reportId")
//    R updateComment(@ApiParam(value = "评语comment和实验报告主键reportId",required = true) ExperimentalReportEntity entity);

    @ApiOperation(value = "发布成绩", notes = "发布成绩--修改教师签名")
    R updateSignatrue(@ApiParam(value = "实验报告主键reportId + 是否使用教师签名isSignature + 教师签名id signatureId + 教师评语comment") ExperimentalReportEntity entity);

    @ApiOperation(value = "成绩管理批量删除项目", notes = "将项目管理下的成绩管理批量删除")
    R deleteReportByIds(@ApiParam(value = "项目id", required = true) List<Integer> projectIds,
                        @ApiParam(value = "项目类型（0练习，1考核）", required = true) Integer projectPermissions,
                        @ApiParam(value = "教学实验id", required = false) List<Integer> ids);


    @ApiOperation(value = "成绩管理:虚拟仿真实验成绩管理", notes = "成绩管理:虚拟仿真实验成绩管理")
    R getImitationAchievement(@ApiParam(value = "projectId", required = false) Integer projectId);

    @ApiOperation(value = "成绩管理:实验教学成绩管理", notes = "成绩管理:实验教学成绩管理")
    R getTeachAchievement(@RequestParam(required = true) Integer id);

    @ApiOperation(value = "成绩管理:编辑实验报告", notes = "成绩管理:编辑实验报告")
    R modify(@ApiParam(value = "要修改的字段") ExperimentalReportEntity entity);

}
