package com.msdw.tms.api;

import com.msdw.tms.common.utils.R;
import com.msdw.tms.entity.request.QuestionsAddRequest;
import com.msdw.tms.entity.request.QuestionsQueryRequest;
import com.msdw.tms.entity.request.QuestionsUpdateRequest;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;


/**
 * 记录试题信息
 *
 * @author gongsj
 */
@Api(value = "试题信息管理", tags = "提供试题信息的增删改查及导入等功能")
public interface QuestionsControllerApi {

    /**
     * 列表
     */
    @ApiOperation(value = "分页加条件查询试题信息", notes = "分页加条件查询试题信息")
    R list(@ApiParam(name = "page", value = "页码", required = true) Integer page,
           @ApiParam(name = "size", value = "每页显示的数据条数", required = true) Integer size,
           QuestionsQueryRequest request);

    /**
     * 信息
     */
    @ApiOperation(value = "根据试题id查询试题详情信息", notes = "根据试题id查询试题详情信息")
    R info(@ApiParam(name = "id", value = "试题主键", required = true) Integer id);

    /**
     * 保存
     */
    @ApiOperation(value = "新增一道试题", notes = "传入新增试题所需相关参数")
    R save(QuestionsAddRequest questions);

    /**
     * 根据试题id修改试题信息
     */
    @ApiOperation(value = "根据试题id修改试题信息", notes = "根据试题id修改试题信息")
    R update(QuestionsUpdateRequest questions);

    /**
     * 是否禁用试题
     */
    @ApiOperation(value = "是否禁用试题", notes = "是否禁用试题")
    R isNable(@ApiParam(name = "id", value = "试题主键", required = true) List<Integer> idList);

    /**
     * 删除
     */
    @ApiOperation(value = "批量删除试题信息", notes = "批量删除试题信息")
    R delete(@ApiParam(name = "ids", value = "试题主键列表", required = true) Integer[] ids);

    /**
     * 通过excel批量导入
     */
    @ApiOperation(value = "通过excel批量导入", notes = "通过excel批量导入")
    R importQuestion(@ApiParam(name = "file", value = "excel文件", required = true) MultipartFile file) throws IOException;

    /**
     * excel模板文件上传
     */
    @ApiOperation(value = "excel模板文件上传", notes = "excel模板文件上传")
    R uploadFiles(@ApiParam(name = "file", value = "excel文件", required = true) MultipartFile file) throws IOException;

    /**
     * excel模板文件下载
     */
    @ApiOperation(value = "excel模板文件下载", notes = "excel模板文件下载")
    R downloadFiles(HttpServletResponse response) throws IOException;

    /**
     * 通过excel批量导出
     */
    @ApiOperation(value = "通过excel批量导出", notes = "通过excel批量导出")
    void exportQuestion(HttpServletResponse response) throws Exception;


    /**
     * 通过excel批量导出
     */
    @ApiOperation(value = "导出导入失败的数据和失败原因", notes = "导出导入失败的数据和失败原因")
    void exportFailureRecord(HttpServletResponse response, String token) throws Exception;

}
