package com.msdw.tms.common.utils;

import com.msdw.tms.common.exception.ExceptionCast;
import com.msdw.tms.entity.response.CommonCode;
import com.msdw.tms.entity.vo.StaffVo;
import com.msdw.tms.entity.vo.StudentVo;
import org.apache.commons.lang.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFRichTextString;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class ExcelImportHelper {


    //判断row是否为空
    public static boolean isRowEmpty(Row row) {
        if (null == row) {
            return true;
        }
        int firstCellNum = row.getFirstCellNum();   //第一个列位置
        int lastCellNum = row.getLastCellNum();     //最后一列位置
        int nullCellNum = 0;    //空列数量
        for (int c = firstCellNum; c < lastCellNum; c++) {
            Cell cell = row.getCell(c);
            if (null == cell || CellType.BLANK == cell.getCellType()) {
                nullCellNum++;
                continue;
            }
            cell.setCellType(CellType.STRING);
            String cellValue = cell.getStringCellValue().trim();
            if (StringUtils.isEmpty(cellValue)) {
                nullCellNum++;
            }
        }
        //所有列都为空
        if (nullCellNum == (lastCellNum - firstCellNum)) {
            return true;
        }
        return false;
    }

    //todo 解决POI无法同时读取带多种格式的单元格表格
    public static String getValue(Cell cell) {
        if (cell == null) {

            return "";

        } else if (cell.getCellType() == CellType.BOOLEAN) {
            return String.valueOf(cell.getBooleanCellValue());
        } else if (cell.getCellType() == CellType.NUMERIC) {
            String value = "";
            return value;
        } else {
            return String.valueOf(cell.getStringCellValue());

        }
    }


    /**
     * 读取学生管理
     *
     * @param file
     * @return
     */
    public static List<StudentVo> readStudent(MultipartFile file) throws IOException {
        List<StudentVo> list = new ArrayList<StudentVo>();

        Workbook workbook = getWorkbook(file);
        StudentVo student = null;
        // 循环工作表Sheet
        for (int numSheet = 0; numSheet < workbook.getNumberOfSheets(); numSheet++) {
            Sheet hssfSheet = workbook.getSheetAt(numSheet);
            if (hssfSheet == null) {
                continue;
            }

            int count = 1;
            // 循环行Row//开始行2
            for (int rowNum = 2; rowNum <= hssfSheet.getLastRowNum(); rowNum++) {
                Row row = hssfSheet.getRow(rowNum);


                if (isRowEmpty(row)) {
                    count++;
                    if (count == hssfSheet.getLastRowNum()) {
                        ExceptionCast.cast(CommonCode.EXCEL_FILE_NULL);
                    }
                    continue;
                }

                if (row == null) {
                    continue;
                } else if (
                        StringUtils.isEmpty(getValue(row.getCell(0))) &&
                                StringUtils.isEmpty(getValue(row.getCell(1))) &&
                                StringUtils.isEmpty(getValue(row.getCell(2)))) {
                    continue;
                }

                Cell phone;
                Cell email;
                if (row != null) {
                    student = new StudentVo();

                    row.getCell(0).setCellType(CellType.STRING);
                    Cell studentName = row.getCell(0);


                    row.getCell(1).setCellType(CellType.STRING);
                    Cell account = row.getCell(1);
                    row.getCell(2).setCellType(CellType.STRING);

                    Cell studentNumber = row.getCell(2);
                    if (row.getCell(3) != null) {
                        row.getCell(3).setCellType(CellType.STRING);
                        phone = row.getCell(3);
                    } else {
                        phone = row.createCell(3);
                    }

                    if (row.getCell(4) != null) {
                        row.getCell(4).setCellType(CellType.STRING);
                        email = row.getCell(4);
                    } else {
                        email = row.createCell(4);
                    }


                    // 学生姓名
                    student.setUserName(studentName.getStringCellValue());
                    // 角色id
                    student.setRoleId(ConstantUtils.STUDENT_ROLE);
                    // 学生学号
                    student.setWorkNumber(studentNumber.getStringCellValue());
                    //用户账号
                    student.setAccount(account.getStringCellValue());
                    //电话
                    student.setPhone(phone.getStringCellValue());
                    //邮箱
                    student.setEmail(email.getStringCellValue());
                    //学校id
                    student.setSchoolId(ConstantUtils.Keda_schoolId);
                    list.add(student);
                }
            }

        }

        return list;
    }


    /**
     * 读取员工管理
     *
     * @param file
     * @return
     */
    public static List<StaffVo> readStaff(MultipartFile file) throws IOException {
        List<StaffVo> list = new ArrayList<StaffVo>();

        Workbook workbook = getWorkbook(file);
        StaffVo staff = null;
        // 循环工作表Sheet
        for (int numSheet = 0; numSheet < workbook.getNumberOfSheets(); numSheet++) {
            Sheet hssfSheet = workbook.getSheetAt(numSheet);
            if (hssfSheet == null) {
                continue;
            }
            /*int lastRowNum = hssfSheet.getLastRowNum() + 1;//最后一行行标，比行数小1
            if (lastRowNum == 2) {
                break;
            }*/
            int count = 1;
            // 循环行Row//开始行2
            for (int rowNum = 2; rowNum <= hssfSheet.getLastRowNum(); rowNum++) {
                Row row = hssfSheet.getRow(rowNum);

                if (isRowEmpty(row)) {
                    count++;
                    if (count == hssfSheet.getLastRowNum()) {
                        ExceptionCast.cast(CommonCode.EXCEL_FILE_NULL);
                    }
                    continue;
                }


                Cell phone;
                Cell email;
                Cell staffGradeName;
                Cell staffProfessionalArchitectureName;
                Cell staffGradeNameTwo;
                Cell staffProfessionalArchitectureNameTwo;
                /*Cell account;
                Cell role;
                Cell workNumber;*/
                Cell userName;


                if (row == null) {
                    continue;
                } else {
                    staff = new StaffVo();

                    if (row.getCell(0) != null) {
                        row.getCell(0).setCellType(CellType.STRING);
                        userName = row.getCell(0);
                    } else {
                        userName = row.createCell(0);
                    }

                    //姓名
                   /* row.getCell(0).setCellType(CellType.STRING);
                    String userName = row.getCell(0).getStringCellValue();*/
                    //账号
                    row.getCell(1).setCellType(CellType.STRING);
                    Cell account = row.getCell(1);
                    //角色
                    row.getCell(2).setCellType(CellType.STRING);
                    Cell role = row.getCell(2);
                    //工号
                    row.getCell(3).setCellType(CellType.STRING);
                    Cell workNumber = row.getCell(3);


                    if (row.getCell(4) != null) {
                        row.getCell(4).setCellType(CellType.STRING);
                        staffProfessionalArchitectureName = row.getCell(4);
                    } else {
                        staffProfessionalArchitectureName = row.createCell(4);
                    }
                    if (row.getCell(5) != null) {
                        row.getCell(5).setCellType(CellType.STRING);
                        staffGradeName = row.getCell(5);
                    } else {
                        staffGradeName = row.createCell(5);
                    }
                    if (row.getCell(6) != null) {
                        row.getCell(6).setCellType(CellType.STRING);
                        staffProfessionalArchitectureNameTwo = row.getCell(6);
                    } else {
                        staffProfessionalArchitectureNameTwo = row.createCell(6);
                    }
                    if (row.getCell(7) != null) {
                        row.getCell(7).setCellType(CellType.STRING);
                        staffGradeNameTwo = row.getCell(7);
                    } else {
                        staffGradeNameTwo = row.createCell(7);
                    }
                    if (row.getCell(8) != null) {
                        row.getCell(8).setCellType(CellType.STRING);
                        phone = row.getCell(8);
                    } else {
                        phone = row.createCell(8);
                    }
                    if (row.getCell(9) != null) {
                        row.getCell(9).setCellType(CellType.STRING);
                        email = row.getCell(9);
                    } else {
                        email = row.createCell(9);
                    }

                    /*row.getCell(10).setCellType(CellType.STRING);
                    Cell schoolAppellationName = row.getCell(10);*/

                    String roleValue = role.getStringCellValue();
                    String str;
                    // 角色id
                    if (roleValue.equals("管理员")) {
                        str = "13";//2
                    } else if (roleValue.equals("老师")) {
                        str = "14";//3
                    } else {
                        str = "13,14";
                    }
                    staff.setRoleId(str);
                    //角色名称
                    staff.setRole(role.getStringCellValue());
                    // 学生姓名
                    staff.setUserName(userName.getStringCellValue());
                    //用户账号
                    staff.setAccount(account.getStringCellValue());
                    // 学生学号
                    staff.setWorkNumber(workNumber.getStringCellValue());
                    //管理员的专业名称
                    staff.setStaffGradeName(staffGradeName.getStringCellValue());
                    //管理员的组织架构名称
                    staff.setStaffProfessionalArchitectureName(staffProfessionalArchitectureName.getStringCellValue());
                    //老师的专业名称
                    staff.setStaffGradeNameTwo(staffGradeNameTwo.getStringCellValue());
                    //老师的组织架构名称
                    staff.setStaffProfessionalArchitectureNameTwo(staffProfessionalArchitectureNameTwo.getStringCellValue());
                    //电话
                    staff.setPhone(phone.getStringCellValue());
                    //邮箱
                    staff.setEmail(email.getStringCellValue());
                    //所属院校
                    //staff.setSchoolAppellationName(schoolAppellationName.getStringCellValue());
                    //学校id
                    staff.setSchoolId(ConstantUtils.Keda_schoolId);
                    list.add(staff);
                }
            }

        }
        return list;
    }

    private static Workbook getWorkbook(MultipartFile file) {
        String fileName = file.getOriginalFilename();
        Workbook workbook = null;

        if (fileName.endsWith("xlsx")) {
            try {
                workbook = new XSSFWorkbook(file.getInputStream());
            } catch (IOException e) {
                e.printStackTrace();
            }
        } else if (fileName.endsWith("xls")) {
            try {
                workbook = new HSSFWorkbook(file.getInputStream());
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return workbook;
    }
}
