package com.msdw.tms.controller;

import com.github.pagehelper.PageInfo;
import com.msdw.tms.api.AchievementManagementApi;
import com.msdw.tms.common.utils.R;
import com.msdw.tms.entity.ExperimentalReportEntity;
import com.msdw.tms.entity.ProjectRecordEntity;
import com.msdw.tms.entity.ReportEntity;
import com.msdw.tms.entity.ResultsEntity;
import com.msdw.tms.entity.vo.AchievementManagementVO;
import com.msdw.tms.entity.vo.ResultsVo;
import com.msdw.tms.service.AchievementManagementService;
import com.msdw.tms.service.ProjectRecordService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 成绩管理
 */
@CrossOrigin
@RestController
@RequestMapping("/Achievement")
public class AchievementManagementController implements AchievementManagementApi {

    @Autowired
    private AchievementManagementService service;

    @Autowired
    private ProjectRecordService projectRecordService;

    /**
     * 查询用户实验报告
     *
     * @param projectId
     * @param
     * @return
     */
    @Override
    @GetMapping("/getTeacher")
    public R getTeacher(@RequestParam Integer projectId) {
        R data = service.getTeacher(projectId);
        return data;
    }

    /**
     * 查询用户实验报告
     *
     * @param userId
     * @param reportId
     * @param
     * @return
     */
    @Override
    @GetMapping("/getReport")
    public R getReport(@RequestParam Integer reportId, @RequestParam Integer userId) {
        R data = service.getReport(userId, reportId);
        return data;
    }


    /**
     * 查询实验记录
     */
    @Override
    @GetMapping("/list")
    public R list(@RequestParam(required = true) Integer systemId,
                  @RequestParam(required = false) Integer projectPermissions,
                  @RequestParam(required = false) String searchContant,
                  @RequestParam(required = false) String startTime,
                  @RequestParam(required = false) String endTime,
                  @RequestParam(required = false) Integer month,
                  @RequestParam(required = true) Integer page,
                  @RequestParam(required = true) Integer size) {
        ResultsVo vo = new ResultsVo();
        if (systemId == null) {
            return R.error("系统id不能为空！");
        } else {
            vo.setSystemId(systemId);
        }
        if (projectPermissions != null) {
            vo.setProjectPermissions(projectPermissions);
        }
        if (searchContant != null) {
            vo.setSearchContant(searchContant);
        }
        if (startTime != null) {
            vo.setStartTime(startTime);
        }
        if (endTime != null) {
            vo.setEndTime(endTime);
        }
        PageInfo<ResultsEntity> result = service.list(vo, page, size);


        return R.ok().put("data", result);
    }

    /**
     * 查询实验教学记录
     */
    @Override
    @GetMapping("/newList")
    public R newList(@RequestParam(required = true) Integer userId,
                     @RequestParam(required = false) String searchContant,
                     @RequestParam(required = false) String startTime,
                     @RequestParam(required = false) String endTime,
                     @RequestParam(required = false) Integer month,
                     @RequestParam(required = true) Integer systemId,
                     @RequestParam(required = true) Integer page,
                     @RequestParam(required = true) Integer size) {
        ResultsVo vo = new ResultsVo();
        if (userId == null) {
            return R.error("用户id不能为空！");
        } else {
            vo.setUserId(userId);
        }
        if (searchContant != null) {
            vo.setSearchContant(searchContant);
        }
        if (startTime != null) {
            vo.setStartTime(startTime);
        }
        if (endTime != null) {
            vo.setEndTime(endTime);
        }
        if (systemId != null){
            vo.setSystemId(systemId);
        }
        PageInfo<ResultsEntity> result = service.newList(vo, page, size);
        return R.ok().put("data", result);
    }

    /**
     * 查询实验记录
     */
    @Override
    @GetMapping("/reportList")
    public R reportList(@RequestParam(required = true) Integer systemId,
                        @RequestParam(required = true) Integer projectId,
                        @RequestParam(required = false) String searchContant,
                        @RequestParam(required = true) Integer page,
                        @RequestParam(required = true) Integer size) {
        ResultsVo vo = new ResultsVo();
        if (systemId == null || projectId == null) {
            return R.error("系统或项目数据不能为空！");
        } else {
            vo.setSystemId(systemId);
            vo.setProjectId(projectId);
        }
        if (searchContant != null) {
            vo.setSearchContant(searchContant);
        }
        List<ReportEntity> result = service.reportList(vo, page, size);
        return R.ok().put("data", result);
    }

    /**
     * 提交实验结果--川大
     */
    @Override
    @PostMapping("/addReport")
    public R addReport(@RequestBody ExperimentalReportEntity entity) {
        R result = service.addReport(entity);
        return R.ok().put("reportId", entity.getReportId());
    }

//    /**
//     * 提交实验报告
//     */
//    @Override
//    @PostMapping("/add")
//    public R add(@RequestBody ExperimentalProjectEntity vo){
//        R result = service.addReport(vo);
//        return result;
//    }

    /**
     * 查询实验记录
     */
    @Override
    @GetMapping("/queryReport")
    public R queryReport(@RequestParam(required = true) Integer reportId) {
        R result = service.queryReport(reportId);
        return result;
    }

    //直接修改实验记录分数
//    @Override
    @Transactional
    @PostMapping("/updateReport")
    public R updateScore(@RequestBody ProjectRecordEntity entity) {
        ProjectRecordEntity vo = new ProjectRecordEntity()
                .setRecordid(entity.getRecordid()).setScore(entity.getScore());
        projectRecordService.updateScore(vo);
        return R.ok();
    }

    /**
     * 需求：删除实验记录====管理端删除，但是学生端仍需保留
     * 将isdel属性设置为0
     *
     * @param reportId
     * @return
     */
    @Override
    @PostMapping("/deleteReport")
    public R deleteReport(@RequestParam List<Integer> reportId, @RequestParam List<Integer> recordId) {
        if (reportId == null || reportId.size() == 0){
            return R.error(400,"参数错误!");
        }
        if (recordId == null || recordId.size() == 0){
            return R.error(400,"参数错误!");
        }
        if (reportId.size() != recordId.size()){
            return R.error(400,"参数错误!");
        }
        R result = service.deleteReport(reportId, recordId);
        return result;
    }


    /**
     * 班级实验报告列表查询
     *
     * @param searchContant
     * @return
     */
    @Override
    @GetMapping("/queryEvaluationReport")
    public R queryEvaluationReport(@RequestParam(required = false) String searchContant,
                                   @RequestParam(required = true) Integer projectId,
                                   @RequestParam(required = true) Integer page,
                                   @RequestParam(required = true) Integer size) {
        R result = service.queryEvaluationReport(searchContant, projectId, page, size);
        return result;
    }

    /**
     * 班级实验成绩导出
     *
     * @param response
     * @param ids      (ids：recordId)
     * @throws Exception
     */
    @Override
    @GetMapping("/exportAchievement")
    public void exportAchievement(HttpServletResponse response, @RequestParam String ids, @RequestParam Integer source) throws Exception {
        service.exportAchievement(response, ids, source);//SearchAchievementVo vo
    }

    //成绩管理列表
    @Override
    @GetMapping("/queryAchievement")
    public R queryAchievement(@RequestParam(required = false) Integer systemId,
                              @RequestParam(required = false) Integer projectPermissions,
                              @RequestParam(required = false) String searchContant,
                              @RequestParam(required = false) String startingtime,
                              @RequestParam(required = false) String endtime,
                              @RequestParam(required = false) Integer month) {
        R result = service.queryAchievement(systemId, projectPermissions, searchContant, startingtime, endtime, month);
        return result;
    }

    /**
     * 修改评语
     *
     * @param entity
     * @return
     */
//    @Override
    @PostMapping("/updateComment")
    public R updateComment(@RequestBody ExperimentalReportEntity entity) {
        R result = service.updateComment(entity);
        return result;
    }

    /**
     * 发布成绩：
     * 修改教师签名+修改教师评语
     *
     * @param entity （isSignature，signatureId，reportId，comment）
     * @return
     */
    @Override
    @PostMapping("/release")
    public R updateSignatrue(@RequestBody ExperimentalReportEntity entity) {
        service.updateSignatrue(entity);
        return R.ok();
    }


    /**
     * 批量删除成绩管理中的项目
     *
     * @return
     */
    @Override
    @PostMapping("/deleteReportById")
    public R deleteReportByIds(@RequestParam List<Integer> projectIds, @RequestParam Integer projectPermissions,
                               @RequestParam(required = false) List<Integer> ids) {
        return service.deleteReportById(projectIds, projectPermissions, ids);
    }

    /**
     * 成绩管理:虚拟仿真实验成绩管理
     */
    @Override
    @GetMapping("/getImitationAchievement")
    public R getImitationAchievement(@RequestParam(required = true) Integer projectId) {
        List<AchievementManagementVO> result = service.getImitationAchievement(projectId);
        return R.ok().put("data", result);
    }

    /**
     * 成绩管理:实验教学成绩管理
     */
    @Override
    @GetMapping("/getTeachAchievement")
    public R getTeachAchievement(@RequestParam(required = true) Integer id) {
        List<AchievementManagementVO> result = service.getTeachAchievement(id);
        return R.ok().put("data", result);
    }


    @Override
    @PostMapping("/modify")
    public R modify(@RequestBody ExperimentalReportEntity entity) {
        Integer code = service.modify(entity);
        return R.ok();
    }
}
