package com.msdw.tms.controller;

import com.msdw.tms.api.ClassTeachingApi;
import com.msdw.tms.common.utils.PageUtils;
import com.msdw.tms.common.utils.R;
import com.msdw.tms.entity.ExperimentalReportEntity;
import com.msdw.tms.entity.ExperimentalStudentEntity;
import com.msdw.tms.entity.ExperimentalTeachingEntity;
import com.msdw.tms.entity.vo.ExperimentalReportVo;
import com.msdw.tms.entity.vo.ProjectRecordVo;
import com.msdw.tms.service.ClassTeachingService;
import com.msdw.tms.service.ExperimentalReportService;
import com.msdw.tms.service.ExperimentalStudentService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("tms/classTech")
public class ClassTeachingController implements ClassTeachingApi {

    @Autowired
    private ClassTeachingService classTeachingService;
    @Autowired
    private ExperimentalStudentService experimentalStudentService;
    @Autowired
    private ExperimentalReportService experimentalReportService;


    /**
     * 用户端查看班级实验列表
     * @param page
     * @param size
     * @param month
     * @param startTime
     * @param endTime
     * @param status
     * @param condition
     * @return
     */
    @Override
    @GetMapping("userRecord")
    public R queryTestRecord(@RequestParam Integer page, @RequestParam Integer size, Integer month, String startTime, String endTime,
                             String condition, @RequestParam Integer status, @RequestParam Integer schoolId) {
        ProjectRecordVo recordVo = new ProjectRecordVo();
        if (status == 1 || status == 2 || status == 3) {//限定实验状态
            recordVo.setStatus(status);
        } else {//不限定实验状态
            recordVo.setStatus(null);
        }
        if (!StringUtils.isEmpty(condition)) {
            recordVo.setCondition(condition);
        }
        // 设置固定时间(当自定义时间为空时)
        if (!StringUtils.isEmpty(month) && StringUtils.isEmpty(startTime) && StringUtils.isEmpty(endTime)) {
            recordVo.setMonth(month);
        }
        // 设置自定义时间
        if (!StringUtils.isEmpty(startTime) || !StringUtils.isEmpty(endTime)) {
            recordVo.setStartTime(startTime);
            recordVo.setEndTime(endTime);
        }
        /*else {
            recordVo.setMonth(null);
            if (!StringUtils.isEmpty(startTime)) {
                recordVo.setStartTime(startTime);
            }
            if (!StringUtils.isEmpty(endTime)) {
                recordVo.setEndTime(endTime);
            }
        }*/
        recordVo.setSchoolId(schoolId);
        PageUtils page1 = classTeachingService.queryClassRecord(page, size, recordVo);
        return R.ok().put("page", page1);
    }

    /**
     * 校验是否需要输入邀请码
     *
     * @param userId
     * @param projectId
     * @return
     */
    @Override
    @GetMapping("/checkInvitationCode")
    public R check(@RequestParam Integer userId, @RequestParam("id") Integer projectId) {
        List o = experimentalStudentService.queryIsCode(userId, projectId);
        if (o.size() > 0) {
            return R.ok();
        } else {
            return R.error(200, "false");
        }
    }

    /**
     * 通过邀请码进入实验
     *
     * @param
     * @return
     */
    @Override
    @Transactional
    @PostMapping("/joinPractice")
    public R joinPractice(@RequestBody ExperimentalTeachingEntity entity1) {
        //获取参数
        Integer projectId = entity1.getProjectId();
        Integer id = entity1.getId();
        Integer invitationCode = entity1.getInvitationCode();
        Integer userId = entity1.getUserId();
        //用于接受参数
        ExperimentalStudentEntity studentEntity = new ExperimentalStudentEntity();
        ExperimentalTeachingEntity entity = new ExperimentalTeachingEntity();
//        Integer ICode = experimentalStudentService.queryIsCode(userId);//获取iscode信息,

        //if (isCode==0)
        if (invitationCode != null) {
            entity.setId(id);
//                entity.setProjectId(projectId);
            ExperimentalTeachingEntity result = classTeachingService.queryInvitationcode(entity);//邀请码唯一
            Integer code = result.getInvitationCode();
            if (invitationCode.equals(code) == true) {
                ExperimentalStudentEntity student = studentEntity.setUserId(userId).setProjectId(id);
                experimentalStudentService.saveCode(student);
                return R.ok();
            } else {
                return R.error("邀请码错误，验证失败！！！");
            }
        } else {
            return R.error(400, "邀请码格式错误！！！");
        }
    }

    /**
     * 查看虚拟仿真课程列表信息
     *
     * @param page
     * @param size
     * @return
     */
    @Override
    @GetMapping("/simulationPlayList")
    public R querySimulationPlayList(@RequestParam Integer page, @RequestParam Integer size) {
        PageUtils page1 = classTeachingService.querySimulationPlayList(page, size);
        return R.ok().put("data", page1);
    }

    /**
     * 学生查看实验报告(教学)
     *
     * @param
     * @return
     */
    @Override
    @GetMapping("/queryExperimentalReport")
    public R queryExperimentalReport(@RequestParam Integer studentId, @RequestParam Integer projectId, @RequestParam Integer recordId, @RequestParam Integer reportId) {
        ExperimentalReportVo entity = new ExperimentalReportVo();
        R result = experimentalReportService.queryReport(studentId, projectId, recordId, reportId);

        return result;
    }

    /**
     * 学生查看虚拟实验报告(虚仿)
     *
     * @param
     * @return
     */
    @Override
    @GetMapping("/queryVirtualReport")
    public R queryVirtualReport(@RequestParam Integer recordId) {
        R result = experimentalReportService.queryVirtualReport(recordId);
        return result;
    }
}
