package com.msdw.tms.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.msdw.tms.api.ExperimentalTeachingControllerApi;
import com.msdw.tms.common.utils.Constant;
import com.msdw.tms.common.utils.PageUtils;
import com.msdw.tms.common.utils.R;
import com.msdw.tms.entity.ExperimentalTeachingEntity;
import com.msdw.tms.entity.vo.ExperimentalTeachingVO;
import com.msdw.tms.service.ExperimentalTeachingService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;

@RestController
@RequestMapping("tms/exp")
public class ExperimentalTeachingController implements ExperimentalTeachingControllerApi {

    @Autowired
    private ExperimentalTeachingService experimentalTeachingService;


    /**
     * 根据id查询实验班级详情
     */
    @Override
    @GetMapping("/getById/{id}")
    public R getById(@PathVariable("id") Integer id) {
        ExperimentalTeachingEntity et = experimentalTeachingService.getById(id);
        return R.ok().put("ExperimentalTeaching", et);
    }

    /**
     * 根据邀请码查询
     */
    @Override
    @GetMapping("/getInvitationCode")
    public R getInvitationCode(@RequestParam Integer invitationCode) {
        ExperimentalTeachingEntity one = experimentalTeachingService.getOne(new QueryWrapper<ExperimentalTeachingEntity>().eq("invitation_code", invitationCode)
                .eq("is_del", Constant.IsDel.NOT_DEL));
        return R.ok().put("InvitationCode", one);
    }

    /**
     * 查询
     */
    @Override
    @GetMapping("/list")
    public R list(@RequestParam Integer page, @RequestParam Integer size,
                  Integer month, Integer type, Integer status, Integer systemId, String startTime,
                  String stopTime, String searchContent, @RequestParam Integer schoolId) {
        ExperimentalTeachingVO vo = new ExperimentalTeachingVO();
        vo.setSchoolId(schoolId);
        if (month != null) {
            vo.setMonth(month);
        }
        if (systemId != null) {
            vo.setSystemId(systemId);
        }
        if (type != null) {
            vo.setType(type);
        }
        if (status != null) {
            vo.setStatus(status);
        }
        if (startTime != null && startTime != "") {
            vo.setStartTime(startTime);
        }
        if (stopTime != null && startTime != "") {
            vo.setStopTime(stopTime);
        }
        if (searchContent != null && searchContent != "") {
            vo.setSearchContent(searchContent);
        }
        PageUtils list = experimentalTeachingService.queryExperimentalTeaching(page, size, vo);
        return R.ok().put("list", list);
    }

    /**
     * 保存
     */
    @Override
    @PostMapping("/save")
    public R save(@RequestBody ExperimentalTeachingEntity experimentalTeachingEntity) {
        experimentalTeachingService.save(experimentalTeachingEntity);
        return R.ok();
    }

    /**
     * 修改
     */
    @Override
    @PostMapping("/update")
    public R update(@RequestBody ExperimentalTeachingEntity experimentalTeachingEntity) throws ParseException {
        //添加实验结束时间
        ExperimentalTeachingEntity byId = experimentalTeachingService.getById(experimentalTeachingEntity.getId());
        if (byId.getType() == 1) {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            String time = byId.getExperimentDuration();
            int d = time.indexOf("d");
            int h = time.indexOf("h");
            int m = time.indexOf("m");
            int dd = Integer.parseInt(time.substring(0, d));
            int hh = Integer.parseInt(time.substring(d + 1, h));
            int mm = Integer.parseInt(time.substring(h + 1, m));
            int newTime = dd * 24 * 60 + hh * 60 + mm;
            Calendar nowTime = Calendar.getInstance();
            Date date = new Date();
            nowTime.setTime(date);
            nowTime.add(Calendar.MINUTE, newTime);
            experimentalTeachingEntity.setStopTime(sdf.format(nowTime.getTime()));
        }
        experimentalTeachingService.updateById(experimentalTeachingEntity);

        return R.ok();
    }

    /**
     * 删除
     */
    @Override
    @PostMapping("/delete")
    public R delete(@RequestBody Integer[] ids) {
        //改为假删除tms_experimental_teaching
        experimentalTeachingService.deleteTeachByIds(Arrays.asList(ids));

        return R.ok();
    }

    @Override
    @GetMapping("/check")
    public R checkExperimentalName(@RequestParam String experimentalName) {
        QueryWrapper<ExperimentalTeachingEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("experimental_name", experimentalName);
        queryWrapper.eq("is_del", 0);
        if (experimentalTeachingService.count(queryWrapper) > 0) {
            return R.error(200, "实验名称不能重复");
        }
        return R.ok();
    }
//    /**
//     * 修改
//     */
//    @PostMapping("/start")
//    public R start(@RequestParam Integer id){
//        experimentalTeachingService.updateById(experimentalTeachingEntity);
//
//        return R.ok();
//    }
}
