package com.msdw.tms.controller;

import com.msdw.tms.api.ProjectRcordControllerApi;
import com.msdw.tms.common.utils.PageUtils;
import com.msdw.tms.common.utils.R;
import com.msdw.tms.entity.ExperimentalReportEntity;
import com.msdw.tms.entity.ProjectEntity;
import com.msdw.tms.entity.ProjectRecordEntity;
import com.msdw.tms.entity.vo.ProjectRecordVo;
import com.msdw.tms.service.ProjectRecordService;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

/**
 * @author
 * @email
 * @date 2020-08-19 16:07:02
 */
@RestController
@RequestMapping("fictitious/projectrecord")
public class ProjectRecordController implements ProjectRcordControllerApi {
    @Autowired
    private ProjectRecordService projectRecordService;

    /**
     * 服务端实验报告
     */
    @Override
    @GetMapping("/schoolRecord")
    public R list(@RequestParam Integer page, @RequestParam Integer size, @RequestParam Integer month, @RequestParam String startTime, @RequestParam String endTime,
                  @RequestParam Integer recordState, @RequestParam String condition) {
        ProjectRecordVo recordVo = new ProjectRecordVo();
        if (!StringUtils.isEmpty(month)) {
            recordVo.setMonth(month);
        }
        if (!StringUtils.isEmpty(startTime)) {
            recordVo.setStartTime(startTime);
        }
        if (!StringUtils.isEmpty(endTime)) {
            recordVo.setEndTime(endTime);
        }
        if (!StringUtils.isEmpty(recordState)) {
            recordVo.setRecordstate(recordState);
        }
        if (!StringUtils.isEmpty(condition)) {
            recordVo.setCondition(condition);
        }
        PageUtils page1 = projectRecordService.querySchoolRecord(page, size, recordVo);

        return R.ok().put("page", page1);
    }

    /**
     * 用户端虚拟仿真实验记录(练习列表)
     *
     * @param page
     * @param size
     * @param
     * @return
     */
    @Override
    @GetMapping("/user/record")
    public R queryUserRecord(@RequestParam Integer page, @RequestParam Integer size, @RequestParam Integer userId, @RequestParam Integer projectPermissions,  @RequestParam Integer systemId) {
        PageUtils page1 = projectRecordService.queryPage(page, size, userId, projectPermissions, systemId);
        return R.ok().put("data", page1);
    }

    /**
     * 科大用户端虚拟仿真实验记录
     *
     * @param page
     * @param size
     * @param
     * @return
     */
    @Override
    @GetMapping("/user/kdRecord")
    public R kdRecord(@RequestParam Integer page, @RequestParam Integer size, @RequestParam Integer userId, @RequestParam Integer projectPermissions, @RequestParam Integer systemId) {
        PageUtils page1 = projectRecordService.getByKDUserRecord(page, size, userId, projectPermissions, systemId);
        return R.ok().put("data", page1);
    }

    /**
     * 用户端教学实验记录（考核列表）
     *
     * @param page
     * @param size
     * @param userid
     * @return
     */
    @Override
    @GetMapping("/user/experimentRecord")
    public R getUserExperimentscore(@RequestParam Integer page, @RequestParam Integer size, @RequestParam Integer userid, @RequestParam Integer systemId) {
        PageUtils page1 = projectRecordService.queryExperimentPage(page, size, userid, systemId);

        return R.ok().put("data", page1);
    }

    /**
     * 个人实验预览
     *
     * @param userId
     * @return
     */
    @Override
    @GetMapping("/user/queryScore/{userId}")
    public R queryScore(@PathVariable("userId") Integer userId) {
        ProjectRecordVo projectRecord = projectRecordService.queryScore(userId);

        return R.ok().put("data", projectRecord);
    }

    /**
     * 个人实验预览
     *
     * @param userid
     * @return
     */
    @Override
    @GetMapping("/user/score/{userId}")
    public R getUserScore(@PathVariable("userId") Integer userid) {
        ProjectRecordVo projectRecord = projectRecordService.queryUserScore(userid);

        return R.ok().put("data", projectRecord);
    }

    /**
     * 导出个人虚拟仿真实验记录
     *
     * @param response
     * @param userId
     * @throws Exception
     */
    @Override
    @GetMapping("/user/exportProjectRecord")
    public void exportProjectRecord(HttpServletResponse response, @RequestParam("userId") Integer userId) throws Exception {
        projectRecordService.exportProjectRecord(response, userId);

    }

    /**
     * 导出个人教学实验记录
     *
     * @param response
     * @param userId
     * @throws Exception
     */
    @Override
    @GetMapping("/user/exportExperimentProjectRecord")
    public void exportExperimentProjectRecord(HttpServletResponse response, @RequestParam("userId") Integer userId) throws Exception {
        projectRecordService.exportExperimentProjectRecord(response, userId);
    }

    /**
     * 信息
     */
    @GetMapping("/info/{recordid}")
    public R info(@PathVariable("recordid") Integer recordid) {
        ProjectRecordEntity projectRecord = projectRecordService.getById(recordid);

        return R.ok().put("projectRecord", projectRecord);
    }

    /**
     * 保存
     */
    @RequestMapping("/save")
    public R save(@RequestBody ProjectRecordEntity projectRecord) {
        projectRecordService.save(projectRecord);

        return R.ok();
    }

    /**
     * 修改实验分数
     */
    @Override
    @PutMapping("/update")
    public R update(@RequestBody ProjectRecordEntity projectRecord) {
        boolean result = projectRecordService.updateUserRecord(projectRecord);

        return R.ok().put("data", result);
    }

    /**
     * 删除实验报告
     */
    @Override
    @DeleteMapping("/delete")
    public R delete(@RequestBody Integer[] recordids) {
        projectRecordService.removeByIds(Arrays.asList(recordids));
        return R.ok();
    }

}
