package com.msdw.tms.controller;

import com.msdw.tms.api.QuestionsControllerApi;
import com.msdw.tms.common.utils.FilesResult;
import com.msdw.tms.common.utils.PageUtils;
import com.msdw.tms.common.utils.R;
import com.msdw.tms.entity.request.QuestionsAddRequest;
import com.msdw.tms.entity.request.QuestionsQueryRequest;
import com.msdw.tms.entity.request.QuestionsUpdateRequest;
import com.msdw.tms.entity.vo.QuestionsDetailVO;
import com.msdw.tms.service.QuestionsService;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;
import java.util.Map;


/**
 * 记录试题信息
 *
 * @author gongsj
 */
@RestController
@RequestMapping("tms/questions")
public class QuestionsController implements QuestionsControllerApi {

    @Resource
    QuestionsService questionsService;

    /**
     * 列表
     */
    @Override
    @GetMapping("/list")
    //@RequiresPermissions("qms:questions:list")
    public R list(@RequestParam Integer page,
                  @RequestParam Integer size,
                  QuestionsQueryRequest request) {

        PageUtils questionsQueryPage = questionsService.queryQuestionsPage(page, size, request);

        return R.ok().put("page", questionsQueryPage);
    }

    /**
     * 信息
     */
    @Override
    @GetMapping("/info/{id}")
    //@RequiresPermissions("qms:questions:info")
    public R info(@PathVariable("id") Integer id) {
        QuestionsDetailVO questions = questionsService.getQuestionDetailById(id);

        return R.ok().put("questions", questions);
    }

    /**
     * 保存
     */
    @ApiOperation(value = "新增一道试题", notes = "传入新增试题所需相关参数")
    @Override
    @PostMapping("/save")
    //@RequiresPermissions("qms:questions:save")
    public R save(@RequestBody QuestionsAddRequest questions) {
        boolean save = questionsService.saveQuestion(questions);

        return save ? R.ok() : R.error();
    }

    /**
     * 根据试题id修改试题信息
     */
    @Override
    @PutMapping("/update")
    //@RequiresPermissions("qms:questions:update")
    public R update(@RequestBody QuestionsUpdateRequest questions) {
        boolean update = questionsService.updateQuestionById(questions);

        return update ? R.ok() : R.error();
    }


    /**
     * 是否禁用试题
     */
    @Override
    @PutMapping("/isenable")
    public R isNable(@RequestBody List<Integer> idList) {
        Integer id = idList.get(0);
        R result = questionsService.isEnable(id);
        return result;
    }
//    }

    /**
     * 删除
     */
    @Override
    @PostMapping("/delete")
    //@RequiresPermissions("qms:questions:delete")
    public R delete(@RequestBody Integer[] ids) {
        boolean delete = questionsService.deleteByIds(Arrays.asList(ids));

        return delete ? R.ok() : R.error();
    }

    /**
     * 通过excel批量导入
     */
    @Override
    @PostMapping("/import")
    //@RequiresPermissions("qms:questions:import")
    public R importQuestion(@RequestParam(name = "file") MultipartFile file) throws IOException {
        Map<String, String> map = questionsService.importQuestion(file);

        return R.ok().put("data", map);
    }

    /**
     * excel模板文件上传
     */
    @Override
    @PostMapping("/upload")
    //@RequiresPermissions("qms:questions:upload")
    public R uploadFiles(MultipartFile file) throws IOException {
        FilesResult filesResult = questionsService.uploadFiles(file);
        return R.ok().put("data", filesResult);
    }

    /**
     * excel模板文件下载
     */
    @Override
    @GetMapping("/download")
    //@RequiresPermissions("qms:questions:download")
    public R downloadFiles(HttpServletResponse response) throws IOException {
        questionsService.downloadFiles(response);
        return R.ok();
    }

    @Override
    @GetMapping("/export")
    public void exportQuestion(HttpServletResponse response) throws Exception {
        questionsService.exportQuestion(response);
    }

    @Override
    @GetMapping("/export_failure")
    public void exportFailureRecord(HttpServletResponse response, String token) throws Exception {
        questionsService.exportFailureRecord(response, token);
    }

}
