package com.msdw.tms.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.msdw.tms.api.StudentControllerApi;
import com.msdw.tms.common.utils.Constant;
import com.msdw.tms.common.utils.ConstantUtils;
import com.msdw.tms.common.utils.PageUtils;
import com.msdw.tms.common.utils.R;
import com.msdw.tms.entity.StudentEntity;
import com.msdw.tms.entity.UserInfoEntity;
import com.msdw.tms.entity.vo.StudentVo;
import com.msdw.tms.service.StudentService;
import com.msdw.tms.service.UserInfoService;
import com.sun.xml.bind.v2.model.core.ID;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

/**
 * 学生管理
 */
@RestController
@RequestMapping("/tms/student")
public class StudentController implements StudentControllerApi{

    @Autowired
    private StudentService studentService;
    @Autowired
    private UserInfoService userInfoService;

    /**
     * 职站新增学生
     * @param studentVo
     * @return
     */
    @Transactional
    @Override
    @PostMapping("/zzAddStudent")
    public R addStudent(@RequestBody @Valid StudentVo studentVo){
        studentVo.setSchoolId(ConstantUtils.Keda_schoolId);
        String account = studentVo.getAccount();
        String phone = studentVo.getPhone();
        Integer schoolId = studentVo.getSchoolId();
        Integer result = studentService.queryStudentIdNumber(studentVo.getWorkNumber(),schoolId);
        Integer res = userInfoService.queryStudentAccount(account,schoolId);
        if (phone!=null&&phone!=""){
            Integer outcome = userInfoService.queryStudentPhone(phone);
            if(outcome==1){
                return R.error("手机号不能重复，只能绑定一个用户");
            }
        }
        String email = studentVo.getEmail();
        if (result==1){
            return R.error("学号不能重复");
        }else if (res==1){
            return R.error("账号不能重复");
        }
        else{
        UserInfoEntity userInfoList = new UserInfoEntity();

        userInfoList.setAccount(account)
                .setPhone(phone)
                .setEmail(studentVo.getEmail())
                .setSchoolId(studentVo.getSchoolAppellationId())
                .setUserName(studentVo.getUserName())
                .setToken(new ConstantUtils().token);
        userInfoService.saveUserInfo(userInfoList);

        studentVo.setUserId(userInfoList.getUserId()).setPlatformId(ConstantUtils.PLATFORMID);//暂时写死平台id（PlatformId）
        boolean saveStudent = studentService.saveStudent(studentVo);
        return saveStudent ?  R.ok() : R.error();
        }
    }

    /**
     * 科大新增学生
     * @param studentVo
     * @return
     */
    @Transactional
    @Override
    @PostMapping("/addStudent")
    public R zzAddStudent(@RequestBody @Valid StudentVo studentVo){
        String account = studentVo.getAccount();
        String phone = studentVo.getPhone();
        Integer schoolId = studentVo.getSchoolId();
        Integer result = studentService.queryStudentIdNumber(studentVo.getWorkNumber(),schoolId);
        Integer res = userInfoService.queryStudentAccount(account,schoolId);
        if (phone!=null&&phone!=""){
            Integer outcome = userInfoService.queryStudentPhone(phone);
            if(outcome==1){
                return R.error("手机号不能重复，只能绑定一个用户");
            }
        }
        String email = studentVo.getEmail();
        if (result==1){
            return R.error("学号不能重复");
        }else if (res==1){
            return R.error("账号不能重复");
        }
        else{
            UserInfoEntity userInfoList = new UserInfoEntity();

            userInfoList.setAccount(account)
                    .setPhone(phone)
                    .setEmail(studentVo.getEmail())
                    .setSchoolId(studentVo.getSchoolAppellationId())
                    .setUserName(studentVo.getUserName())
                    .setToken(new ConstantUtils().token);
            userInfoService.saveUserInfo(userInfoList);

            studentVo.setUserId(userInfoList.getUserId()).setPlatformId(ConstantUtils.PLATFORMID);//暂时写死平台id（PlatformId）
            boolean saveStudent = studentService.saveStudent(studentVo);
            return saveStudent ?  R.ok() : R.error();
        }
    }

    /**
     * 学生列表
     * @param schoolId
     * @param seachContent
     * @param page
     * @param size
     * @return
     */
    @Override
    @GetMapping("/queryStudent")
    public R queryStudent(@RequestParam Integer schoolId,@RequestParam(required = false) String seachContent,@RequestParam Integer page,@RequestParam Integer size){
        PageUtils query = studentService.queryStudent(schoolId,seachContent,page,size);
        return R.ok().put("data",query);
    }

    /**
     * 学生列表
     * @param userId
     * @return
     */
    @Override
    @GetMapping("/queryStudentDetails")
    public R queryStudentDetails(@RequestParam Integer userId){
        R query = studentService.queryStudentDetails(userId);
        return query;
    }

    /**
     * 批量删除学生
     * @param studentId
     * @return
     */
    @Override
    @PostMapping("/daleteStudent")
    public R daleteStudent(@RequestBody List<Integer> studentId){
        boolean delete = studentService.deleteStudent(studentId);

        return delete ? R.ok() : R.error();
    }

    /**
     * 编辑学生
     * @param studentVo
     * @return
     */
    @Override
    @Transactional
    @PutMapping("/updateStudent")
    public R updateStudent(@RequestBody @Valid StudentVo studentVo) {
        studentVo.setSchoolId(ConstantUtils.Keda_schoolId);//设定学校id

        Integer studentId = studentVo.getStudentId();
        String phone = studentVo.getPhone();
        if (studentId==null){
            return R.error(400,"studentId不能为空");
        }

        studentService.updateWorkNumber(studentId);//先将原本的学号重置为00，便于判重
        userInfoService.updateAccount(studentVo.getUserId());//将手机号码/账号重置
        Integer result = studentService.queryStudentIdNumber(studentVo.getWorkNumber(),ConstantUtils.Keda_schoolId);
        Integer res = userInfoService.queryStudentAccount(studentVo.getAccount(),ConstantUtils.Keda_schoolId);
        Integer re = userInfoService.queryStudentPhone(phone);
        if (phone!=null&&phone!=""){
            if (re==1){
                return R.error("手机号码不能重复");
            }
        }
        if (result==1){
            return R.error("学号不能重复");
        }else if (res==1){
            return R.error("账号不能重复");
        }else{
            boolean saveStudent = studentService.updateStudent(studentVo);

            return saveStudent ?  R.ok() : R.error();
        }
    }

    /**
     * 重置密码
     * @param userIds
     * @return
     */
    @Override
    @PutMapping("/reSetPassword")
    public R reSetPassword(@RequestBody List<Integer> userIds){
        Integer userId = userIds.get(0);
        try {
            Integer initial = userInfoService.initialPassword(userId);
            if (initial==0){
                return R.error(400,"密码不变，更新成功");
            }
        }catch (RuntimeException e){
            e.printStackTrace();
            return R.error();
        }
        return R.ok();
    }

    /**
     * 禁用账号
     * @param studentIds
     * @return
     */
    @Override
    @PutMapping("/disableAccount")
    public R disableAccount(@RequestBody StudentVo studentIds) {

        Integer state = studentIds.getDisableAccount();
        if (state==0){
            studentIds.setDisableAccount(1);
        }else if(state==1){
            studentIds.setDisableAccount(0);
        }
        userInfoService.disableAccount(studentIds);
        return R.ok();
    }

    public R downLoadStudent(){
        return R.ok();
    }

    /**
     * excel模板文件下载
     */
    @Override
    @GetMapping("/download")
    public R downloadFiles(HttpServletResponse response) throws IOException {
        studentService.downloadFiles(response);
        return R.ok();
    }

    /**
     * 批量导入学生信息
     * @param file
     * @return
     */
    @Override
    @PostMapping("/uploadFile")
    public R uploadFile(MultipartFile file) throws IOException{
        Integer schoolId = ConstantUtils.Keda_schoolId;
        Map<String,String> upload = studentService.upload(file,schoolId);
        return R.ok().put("data",upload);
    }

    @Override
    @GetMapping("/export_failure")
    public void exportFailureRecord(HttpServletResponse response, String token) throws Exception{

        studentService.exportFailureRecord(response,token);
    }

    /**
     * 查询账号是否存在，若存在则返回账号信息
     * @param account
     * @return
     */
    @Override
    @GetMapping("/queryAccount")
    public R queryAccount(String account,Integer schoolId){
        R result =  studentService.queryAccount(account,schoolId);
        return result;
    }
}
