package com.msdw.tms.controller;


import com.msdw.tms.common.exception.ExceptionCast;
import com.msdw.tms.common.utils.PageUtils;
import com.msdw.tms.entity.SysPermission;
import com.msdw.tms.entity.SysRole;
import com.msdw.tms.entity.SysRolePermission;
import com.msdw.tms.entity.request.RolePermissionRequest;
import com.msdw.tms.entity.response.CommonCode;
import com.msdw.tms.entity.response.ResponseResult;
import com.msdw.tms.entity.response.RolePermissionResponse;
import com.msdw.tms.service.ISysPermissionService;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 菜单权限表 前端控制器
 * </p>
 *
 * @author Qyq
 * @since 2021-02-24
 */
@RestController
@RequestMapping("/sys-permission")
@Api(tags = {"角色权限管理"})
public class SysPermissionController {

    @Autowired
    private ISysPermissionService permissionService;

    /**
     * 权限菜单树形列表展示
     * @return
     */
    @GetMapping("/tree")
    public ResponseResult tree(){
        List<SysPermission> list = permissionService.listWithTree();
        return new ResponseResult<List<SysPermission>>().SUCCESS(list);
    }

    /**
     * 新增角色权限
     */
    @Transactional
    @PostMapping("/saveRolePermission")
    public ResponseResult saveRolePermission(@Valid @RequestBody RolePermissionRequest rolePermission) {
        Date date = new Date();
        rolePermission.setCreateTime(date);
        rolePermission.setMotifyTime(date);
        //此处要判重，新增的角色名称要唯一
        int count = permissionService.uniqueRoleName(rolePermission.getName());
        if(count >0){
            ExceptionCast.cast(CommonCode.ROLE_NAME_EXIST);
        }
        //新增角色到角色表
        int request = permissionService.addHrmsRole(rolePermission);
        //根据选中的permissionId批量新增数据到hrms_role_permission
        List<Long> asList = Arrays.asList(rolePermission.getPermissionIds());
        List<RolePermissionRequest> rolePermissionList = getRolePermissionList(asList, rolePermission.getRoleId());
        boolean result = permissionService.addHrmsRolePermission(rolePermissionList);

        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    private List<RolePermissionRequest> getRolePermissionList(List<Long> asList, Long roleId) {
        return asList.stream().map(permissionId -> {
            RolePermissionRequest request = new RolePermissionRequest();
            request.setRoleId(roleId);
            request.setPermissionId(permissionId);
            return request;
        }).collect(Collectors.toList());
    }

    /**
     * 修改角色权限
     */
    @Transactional
    @PostMapping("/updateRolePermission")
    public ResponseResult updateRolePermission(@Valid @RequestBody RolePermissionRequest rolePermission) {
        Date date = new Date();
        rolePermission.setMotifyTime(date);
        //根据角色Id修改角色内容
        int request = permissionService.updateHrmsRole(rolePermission);
        //删除该角色Id对应的所有权限
        List<Long> asList = Arrays.asList(rolePermission.getPermissionIds());
        permissionService.deleteHrmsRolePermission(rolePermission.getRoleId());
        //根据选中的permissionId批量新增数据到hrms_role_permission
        List<RolePermissionRequest> rolePermissionList = getRolePermissionList(asList, rolePermission.getRoleId());
        boolean result = permissionService.addHrmsRolePermission(rolePermissionList);

        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    /**
     * 删除用户角色权限
     */
    @Transactional
    @PostMapping("/delRolePermission")
    public ResponseResult delRolePermission(@RequestBody RolePermissionRequest rolePermission) {
        //根据角色Id查询该角色是否有用户关联
        int count = permissionService.queryCountByRoleId(rolePermission.getRoleIds());
        if(count >0){
            ExceptionCast.cast(CommonCode.ROLE_EMP_EXIST);
        }
        //没有关联则根据roleId删除hrms_role表对应记录
        boolean b = permissionService.deleteHrmsRoleByRoleId(rolePermission.getRoleIds());
        //根据角色id删除对应所有权限
        boolean result = permissionService.deleteHrmsRolePermission(rolePermission.getRoleIds());

        return result ? ResponseResult.SUCCESS() : ResponseResult.FAIL();
    }

    /**
     * 查看权限数组
     * @param roleId
     * @return
     */
    @GetMapping("/queryPermissionArrById")
    public ResponseResult queryPermissionArrById(@RequestParam("roleId") Long roleId) {
        List<SysRolePermission> list = permissionService.queryPermissionArrById(roleId);
        ArrayList<Integer> sb = new ArrayList<Integer>();
        for (SysRolePermission rolePermission:list) {
            sb.add(rolePermission.getPermissionId().intValue());
        }
        Integer[] ids = sb.toArray(new Integer[sb.size()]);
        //根据roleId查询角色名称和角色描述
        SysRole roleMsg = permissionService.queryRoleMsg(roleId);
        Map map = new HashMap();
        map.put("permissionIds",ids);
        map.put("name",roleMsg.getName());
        map.put("description",roleMsg.getDescription());
        return new ResponseResult<List<RolePermissionResponse>>().SUCCESS(map);
    }

    /**
     * 角色权限列表展示
     * @param page
     * @param size
     * @return
     */
    @GetMapping("/rolePermissionList")
    public ResponseResult rolePermissionList(@RequestParam("page") Integer page,
                                             @RequestParam("size") Integer size,@RequestParam("name")String name) {
        PageUtils data = permissionService.queryPage(page, size, name);
        return new ResponseResult<PageUtils>().SUCCESS(data);
    }

}
