package com.msdw.tms.controller;

import com.msdw.tms.api.SystemSetttingApi;
import com.msdw.tms.common.utils.ConstantUtils;
import com.msdw.tms.common.utils.PageUtils;
import com.msdw.tms.common.utils.R;
import com.msdw.tms.entity.vo.StaffVo;
import com.msdw.tms.service.SystemSetttingService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 系统设置
 */
@RestController
@RequestMapping("/tms/system")
public class SystemSettingController implements SystemSetttingApi {
    @Autowired
    private SystemSetttingService systemSetttingService;

    /**
     * 新增员工
     * @param staffVo
     * @return
     */
    @Override
    @PostMapping("/addStaff")
    public R addStaff(@RequestBody @Valid StaffVo staffVo){
        R r = systemSetttingService.addStaff(staffVo);
        return r;
    }

    /**
     * 员工列表
     * @param staffVo
     * @return
     */
    @Override
    @PostMapping("/queryStaff")
    public R queryStaff(@RequestBody StaffVo staffVo){
        PageUtils query = systemSetttingService.queryStaff(staffVo);
        return R.ok().put("data",query);
    }

    /**
     * 查看用户详情
     * @param userId
     * @return
     */
    @Override
    @GetMapping("/querystaffDetail")
    public R querystaffDetail(@RequestParam Integer userId){
        HashMap<String, Object> map = systemSetttingService.querystaffDetail(userId);
        return R.ok().put("data",map);
    }

    /**
     * 编辑员工信息
     * @param staffVo
     * @return
     */
    @Override
    @PutMapping("/updateStaff")
    public R updateStaff(@RequestBody StaffVo staffVo){
        R r = systemSetttingService.updateStaff(staffVo);
        return r;
    }

    /**
     * 删除员工信息
     * @param staffVo
     * @return
     */
    @Override
    @PostMapping("/deleteStaff")
    public R deleteStaff(@RequestBody StaffVo staffVo){

        systemSetttingService.deleteStaff(staffVo.getUserId());
        return R.ok();
    }

    /**
     * 批量删除员工信息
     * @return
     */
    @Override
    @PostMapping("/daleteBatchStaff")
    public R daleteBatchStaff(@RequestBody List<Integer> userIds){
        systemSetttingService.daleteBatchStaff(userIds);
        return R.ok();
    }

    /**
     * 删除组织架构的同时删除员工（假删）
     * @return
     */
    @GetMapping("/dalStaffByStaffGradeId")
    public R dalStaffByStaffGradeId(@RequestParam("staffGradeId") Integer staffGradeId){
        //根据staffGradeId查询staff表中的userId，然后调用批量删除员工信息接口
        List<Integer> userIds = systemSetttingService.queryUserIdsByGradeId(staffGradeId);
        if(userIds.size()>0){
            systemSetttingService.daleteBatchStaff(userIds);
            systemSetttingService.daleteBatchStaffPs(staffGradeId);
        }else{//如果组织架构下面没有员工，则只删除组织架构
            systemSetttingService.daleteBatchStaffPs(staffGradeId);
        }
        return R.ok();
    }

    /**
     * 删除专业的同时删除员工（假删）
     * @return
     */
    @GetMapping("/dalStaffByProfessionalId")
    public R dalStaffByProfessionalId(@RequestParam("staffProfessionalArchitectureId") Integer staffProfessionalArchitectureId){
        //根据staffGradeId查询staff表中的userId，然后调用批量删除员工信息接口
        List<Integer> userIds = systemSetttingService.queryUserIdsByProfessionalId(staffProfessionalArchitectureId);
        if(userIds.size()>0){
            systemSetttingService.daleteBatchStaff(userIds);
        }
        return R.ok();
    }

    /**
     * 新增部门
     */
    @Override
    @PostMapping("/addDepartment")
    public R addDepartment(@RequestBody StaffVo staffVo){
        HashMap<String, Object> map =  systemSetttingService.addDepartment(staffVo);
        return R.ok().put("data",map);
    }

    /**
     * excel模板文件下载
     */
    @Override
    @GetMapping("/download")
    public R downloadFiles(HttpServletResponse response) throws IOException {
        systemSetttingService.downloadFiles(response);
        return R.ok();
    }

    /**
     * 批量导入员工信息
     * @param file
     * @return
     */
    @Override
    @PostMapping("/uploadFile")
    public R importStaff(MultipartFile file) throws IOException{
        Map<String,String> upload = systemSetttingService.upload(file);
        return R.ok().put("data",upload);
    }

    /**
     * 导入员工信息失败数据导出
     * @param response
     * @param token
     * @throws Exception
     */
    @Override
    @GetMapping("/export_failure")
    public void exportFailureRecord(HttpServletResponse response, @RequestParam String token) throws Exception{
        systemSetttingService.exportFailureRecord(response,token);
    }

    /**
     * 查询账号信息
     * 若存在于本校则返回信息
     * @param account
     * @return
     */
    @Override
    @GetMapping("/queryAccount")
    public R queryAccount(String account,Integer schoolId){
        R result = systemSetttingService.queryAccount(account,schoolId);
        return result;
    }


    /**
     * 重置密码
     * @param userId
     * @return
     */
    @GetMapping("/resetPwd")
    public R resetPwd(@RequestParam Integer userId,String newPwd){
        R r = systemSetttingService.resetPwd(userId,newPwd);
        return r;
    }
}
