package com.msdw.tms.controller;

import com.msdw.tms.api.UserControllerApi;
import com.msdw.tms.common.utils.PageUtils;
import com.msdw.tms.common.utils.R;
import com.msdw.tms.entity.UserEntity;
import com.msdw.tms.entity.UserInfoEntity;
import com.msdw.tms.entity.UserPersonalFileEntity;
import com.msdw.tms.entity.vo.CheckVo;
import com.msdw.tms.entity.vo.UserEntityVo;
import com.msdw.tms.entity.vo.UserProsonalEntityVo;
import com.msdw.tms.service.UserInfoService;
import com.msdw.tms.service.UserService;
//import com.sun.org.apache.bcel.internal.generic.CHECKCAST;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.util.*;

@RestController
@RequestMapping("tms/user")
public class UserController implements UserControllerApi {

    @Autowired
    private UserService userService;
    @Autowired
    private UserInfoService userInfoService;

    /**
     * 列表
     */
    @GetMapping("/list")
    public R list(@RequestParam Map<String, Object> params){
        PageUtils page = userService.queryPage(params);

        return R.ok().put("page", page);
    }


    /**
     * 信息
     */
    @GetMapping("/info/{id}")
    public R info(@PathVariable("id") Long id){
        UserEntity userEntity = userService.getById(id);


        return R.ok();
    }

    @Override
    @GetMapping("/userinfo/{id}")
    public R userInfo(@PathVariable("id") Integer id){

        HashMap<String,UserProsonalEntityVo> userEntityVoList = userService.queryUserInfo(id);
//        UserEntityVo userEntityVo = userService.queryUserId(id);
        return R.ok().put("data",userEntityVoList);
    }

    /**
     * 保存
     */
    @PostMapping("/save")
    public R save(@RequestBody UserEntity userEntity){
        String  token ="UT_"+ UUID.randomUUID().toString().replace("-", "");
        userEntity.setToken(token);
        userService.save(userEntity);

        return R.ok();
    }

    /**
     * 修改
     */
    @Override
    @Transactional
    @PutMapping("/update")
    public R update(@RequestBody UserProsonalEntityVo vo){
        List<UserPersonalFileEntity> entityList = vo.getPersonalFileEntities();
        UserInfoEntity userInfo = vo.getUserInfoEntity();
        Integer userId = userInfo.getUserId();
        String password = userInfo.getPassword();
        String userPassword = userInfoService.queryPasword(userId);
        if (userPassword.equals(password)==true){
            return R.error(400,"false,the password is the same!!!");
        }else{
            userInfoService.updateUserInfoById(userInfo);
//        userInfoService.updateById(userInfo);
            for (int i = 0;i<entityList.size();i++){
                UserPersonalFileEntity personalFile = entityList.get(i);
                Integer personalFileId = personalFile.getPersonalFileId();
                if (personalFile.getPersonalCareerId()==null){
                    return R.error(403,"false,personalCareerId can not be empty!!!");
                }
                if(personalFileId==null){
                    userService.insertPersonalInfo(personalFile);
                }else{
                    userService.updateUserPersonal(personalFile);
                }
            }
            return R.ok();
        }
    }

    /**
     * 删除
     */
    @DeleteMapping("/delete")
    public R delete(@RequestBody Long[] ids){
        userService.removeByIds(Arrays.asList(ids));

        return R.ok();
    }

    /**
     * 校验并更新密码
     * @param entity 封装用户id和输入的原密码和新密码
     * @return R 响应的状态信息
     */
    @Override
    @Transactional
    @PostMapping("/examinePassword")
    public R examinePassword(@RequestBody UserEntityVo entity){
        Integer userid = entity.getUserid();//用户id
        String password = entity.getPassword();//用户输入的当前密码
        String newPassword = entity.getNewPassword();//获取用户输入的新密码
        UserEntity byId = userService.getById(userid);//获取用户个人信息

        UserEntity user = new UserEntity();//用于封装数据
        if (password.equals(byId.getPassword())==true){
            user.setUserid(userid).setPassword(newPassword);
            boolean b = userService.updateById(user);
            if (b==true){
                return R.ok();
            }else{return R.error("更新密码失败，请稍后重试");}
        }else{
            return R.error(403,"原密码错误，更新密码失败！！！");
        }
    }

    /**
     * 发送邮箱验证码
     * @param checkVo (email)
     * @return  R （参数 opener）
     */
    @Override
    @PostMapping("/sendEmailCode")
    public R sendEmailCode(@RequestBody CheckVo checkVo){
        boolean re = userService.queryEmial(checkVo);//查询新邮箱号是否存在重复
        if (re){
            return R.error(403,"邮箱号不能重复");
        }else{
            R save = userService.saveCheckCode(checkVo);
            return save;
        }
    }

    /**
     * 校验邮箱验证码，通过则更新邮箱
     * @param checkVo   （useId 、email 、code 、opener）
     * @return R
     */
    @Override
    @PutMapping("/bingEmail")
    public R bindEmail(@RequestBody CheckVo checkVo) {
        R result = userService.bindEmail(checkVo);
        return result;
    }


        /**
         * 发送手机验证码
         * @param checkVo
         * @return
         */
    @Override
    @PostMapping("/sendPhoneCode")
    public R sendPhoneCode(@RequestBody CheckVo checkVo){
        boolean re = userService.queryPhone(checkVo);
        if (re){
            return R.error(403,"电话号码不能重复");
        }else {
            R result = userService.sendCheckPhoneCode(checkVo);
            return result;
        }
    }

    /**
     * 绑定手机
     * @param checkVo   (userId 、phone、code、opener)
     * @return
     */
    @Override
    @PutMapping("/bindPhone")
    public R bindPhone(@RequestBody CheckVo checkVo){

        R result = userService.bindPhone(checkVo);
        return result;
    }
}
