package com.msdw.tms.controller;

import com.msdw.tms.api.UserControllerApi;
import com.msdw.tms.api.UserInfoControllerApi;
import com.msdw.tms.common.utils.PageUtils;
import com.msdw.tms.common.utils.R;
import com.msdw.tms.entity.UserEntity;
import com.msdw.tms.entity.UserInfoEntity;
import com.msdw.tms.entity.UserPersonalFileEntity;
import com.msdw.tms.entity.vo.CheckVo;
import com.msdw.tms.entity.vo.StudentVo;
import com.msdw.tms.entity.vo.UserEntityVo;
import com.msdw.tms.entity.vo.UserProsonalEntityVo;
import com.msdw.tms.service.StudentService;
import com.msdw.tms.service.UserInfoService;
import com.msdw.tms.service.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.util.*;

@RestController
@RequestMapping("tms/userInfo")
public class UserInfoController implements UserInfoControllerApi {
    @Autowired
    private UserInfoService userInfoService;

    @Autowired
    private StudentService studentService;

    @Autowired
    private UserService userService;

    /**
     * 列表
     */
    @GetMapping("/list")
    public R list(@RequestParam Map<String, Object> params){
        PageUtils page = userInfoService.queryPage(params);

        return R.ok().put("page", page);
    }


    /**
     * 信息
     */
    @GetMapping("/info/{id}")
    public R info(@PathVariable("id") Long id){
        UserInfoEntity userEntity = userInfoService.getById(id);

        return R.ok().put("growthChangeHistory", userEntity);
    }

    /**
     * 注册
     */
    @Override
    @Transactional
    @PostMapping("/add")
    public R save(@RequestBody UserInfoEntity userInfoEntity,@RequestParam String workNumber){
        String  token ="UT_"+ UUID.randomUUID().toString().replace("-", "");
        userInfoEntity.setToken(token);
        Integer schoolId = userInfoEntity.getSchoolId();
        Integer integer = userInfoService.queryStudentAccount(userInfoEntity.getAccount(),schoolId);
        Integer integer1 = userInfoService.queryStudentPhone(userInfoEntity.getPhone());

        //新增学生
        StudentVo studentVo = new StudentVo();

        studentVo.setRoleId(Integer.valueOf(userInfoEntity.getRoleId()));
        studentVo.setSchoolId(schoolId);
        studentVo.setIsdel(0);
        studentVo.setWorkNumber(workNumber);
        studentVo.setPlatformId(1);
        //为1 学号有重复
        int num = studentService.queryStudentByWorkNumber(studentVo);

        if (integer==1){
            return R.error(400,"账号已存在");
        }
        if (integer1==1){
            return R.error(400,"该手机号已被绑定");
        }
        if (num == 1){
            return R.error(400,"该学号重复！");
        }
        int userId = userInfoService.add(userInfoEntity);
        if (userId==0){
            return R.error(400,"添加用户信息失败");
        }
//        userInfoEntity.setUserId(userId);
        studentVo.setUserId(userInfoEntity.getUserId());
        return studentService.addStudentInfo(studentVo);
    }

    /**
     * 职站注册
     */
    @Override
    @Transactional
    @PostMapping("/zzAdd")
    public R zzAdd(@RequestBody UserInfoEntity userInfoEntity,@RequestParam String workNumber){
        String  token ="UT_"+ UUID.randomUUID().toString().replace("-", "");
        userInfoEntity.setToken(token);
        userInfoEntity.setSchoolId(2105);
        Integer schoolId = userInfoEntity.getSchoolId();
        Integer integer = userInfoService.queryStudentAccount(userInfoEntity.getAccount(),schoolId);
        Integer integer1 = userInfoService.queryStudentPhone(userInfoEntity.getPhone());

        //新增学生
        StudentVo studentVo = new StudentVo();

        studentVo.setRoleId(Integer.valueOf(userInfoEntity.getRoleId()));
        studentVo.setSchoolId(schoolId);
        studentVo.setIsdel(0);
        studentVo.setWorkNumber(workNumber);
        studentVo.setPlatformId(1);
        //为1 学号有重复
        int num = studentService.queryStudentByWorkNumber(studentVo);
        if (integer==1){
            return R.error(400,"账号已存在");
        }
        if (integer1==1){
            return R.error(400,"该手机号已被绑定");
        }
        if (num == 1){
            return R.error(400,"该学号重复！");
        }
        int userId = userInfoService.add(userInfoEntity);
        if (userId==0){
            return R.error(400,"添加用户信息失败");
        }
//        userInfoEntity.setUserId(userId);
        studentVo.setUserId(userInfoEntity.getUserId());
        return studentService.addStudentInfo(studentVo);
    }

    /**
     * 修改
     */
//    @Override
    @PutMapping("/update")
    public R update(@RequestBody UserInfoEntity userEntity){
        userInfoService.updateById(userEntity);

        return R.ok();
    }

    /**
     * 删除
     */
    @DeleteMapping("/delete")
    public R delete(@RequestBody Long[] ids){
        userInfoService.removeByIds(Arrays.asList(ids));

        return R.ok();
    }

    /**
     * 查询个人信息
     * @param id
     * @return
     */
    @Override
    @GetMapping("/userinfo/{id}")
    public R userInfo(@PathVariable("id") Integer id){

        HashMap<String,UserProsonalEntityVo> userEntityVoList = userService.queryUserInfo(id);
        return R.ok().put("data",userEntityVoList);
    }

    /**
     * 修改
     */
    @Override
    @Transactional
    @PutMapping("/updateUser")
    public R updateUser(@RequestBody UserProsonalEntityVo vo){
        List<UserPersonalFileEntity> entityList = vo.getPersonalFileEntities();
        UserInfoEntity userInfo = vo.getUserInfoEntity();
        Integer userId = userInfo.getUserId();
        String password = userInfo.getPassword();

        //验证账号是否存在
        int num = userInfoService.queryAccountExist(userInfo.getAccount(),userInfo.getUserId());
        if (num >= 1){
            return R.error(500,"更新失败，账号不能重复");
        }
        String userPassword = userInfoService.queryPasword(userId);
        if (userPassword.equals(password)==true){
            return R.error(400,"false,the password is the same!!!");
        }else{
            userInfoService.updateUserInfoById(userInfo);
            for (int i = 0;i<entityList.size();i++){
                UserPersonalFileEntity personalFile = entityList.get(i);
                Integer personalFileId = personalFile.getPersonalFileId();
                if (personalFile.getPersonalCareerId()==null){
                    return R.error(403,"false,personalCareerId can not be empty!!!");
                }
                if(personalFileId==null){
                    userService.insertPersonalInfo(personalFile);
                }else{
                    userService.updateUserPersonal(personalFile);
                }
            }
            return R.ok();
        }
    }

    /**
     * 找回密码---发送验证码---手机
     * @param phone
     * @return
     */
    @Override
    @GetMapping("/findPasswordByPhone")
    public R findPasswordByPhone(@RequestParam String phone){
        R result = userInfoService.findPasswordByPhone(phone);
        return result;
    }
    /**
     * 找回密码---发送验证码---手机
     * @param email
     * @return
     */
    @Override
    @GetMapping("/findPasswordByEmail")
    public R findPasswordByEmail(@RequestParam String email){
        R result = userInfoService.findPasswordByEmail(email);
        return result;
    }

    /**
     * 校验验证码
     * @param checkVo   （opener 、 code）
     * @return
     */
    @Override
    @PostMapping("/checkCode")
    public R checkCode(@RequestBody CheckVo checkVo){
        R result =userInfoService.checkCode(checkVo);
        return result;
    }

    /**
     * 设置密码
     * @param checkVo   （userId 、 password）
     * @return
     */
    @Override
    @PostMapping("/resetPassword")
    public R resetPassword(@RequestBody CheckVo checkVo){
        R result = userInfoService.resetPassword(checkVo);
        return result;
    }

    @Override
    @GetMapping("/GMAccount")
    public R GMAccount(){
        R result = userInfoService.queryAccount();
        return result;
    }

    /**
     * 根据token获取用户id
     */
    @Override
    @GetMapping("/getUserId")
    public R getUserId(@RequestParam String token){
        R result = userInfoService.getUserId(token);
        return result;
    }
}
