package com.msdw.tms.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.msdw.tms.common.utils.Constant;
import com.msdw.tms.common.utils.PageUtils;
import com.msdw.tms.common.utils.R;
import com.msdw.tms.common.utils.poi.ExcelExportUtil;
import com.msdw.tms.dao.*;
import com.msdw.tms.entity.*;
import com.msdw.tms.entity.request.AchievementImportRequest;
import com.msdw.tms.entity.request.ImportAssessmentRequest;
import com.msdw.tms.entity.vo.AchievementManagementVO;
import com.msdw.tms.entity.vo.JudgmentPointsRuleVO;
import com.msdw.tms.entity.vo.ResultsVo;
import com.msdw.tms.entity.vo.UserScoreVO;
import com.msdw.tms.service.AchievementManagementService;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.formula.functions.T;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletResponse;
import java.io.InputStream;
import java.util.*;

@Slf4j
@Service
public class AchievementManagementServiceImpl extends ServiceImpl<AchievementManagementDao, ExperimentalReportEntity> implements AchievementManagementService {

    @Autowired
    private AchievementManagementDao achievementManagementDao;

    @Autowired
    private ProjectRecordDao projectRecordDao;

    @Autowired
    private ExperimentalTeachingDao experimentalTeachingDao;

    @Autowired
    private UserInfoDao userInfoDao;

    @Autowired
    private ProjectHiddenDao projectHiddenDao;

    //查询实验报告
    @Override
    @Transactional
    public R getReport(Integer userId, Integer reportId) {
        HashMap<String, Object> obj = new HashMap<String, Object>();
        HashMap<String, Object> map = new HashMap<String, Object>();
        UserScoreVO userScoreVO = new UserScoreVO();
        userScoreVO.setUserId(userId);
        userScoreVO.setReportId(reportId);
        ExperimentalReportEntity experimentalReportEntity = new ExperimentalReportEntity();
        experimentalReportEntity.setStudentId(userId);
        experimentalReportEntity.setReportId(reportId);
        ExperimentalReportEntity experimentalReportEntity1 = baseMapper.getReport(experimentalReportEntity);
        List<UserScoreVO> score = baseMapper.getScore(userScoreVO);
        obj.put("score", score);
        obj.put("experimentalReportEntity", experimentalReportEntity1);
        List<Integer> tradingJudgmentPointsIds = baseMapper.queryTradingJudgmentPointsId(reportId);
        Integer size = tradingJudgmentPointsIds.size();
        for (int i = 0; i < size; i++) {
            Integer tradingJudgmentPointsId = tradingJudgmentPointsIds.get(i);
            List<JudgmentPointsRuleVO> judgmentPointsRuleList = baseMapper.getUserAnswers(tradingJudgmentPointsId);
            map.put("judgmentPointsRuleList" + i, judgmentPointsRuleList);
        }
        obj.put("userAnswersList", map);
        return R.ok().put("data", obj);
    }

    @Override
    public R getTeacher(Integer projectId) {
        String name = baseMapper.getTeacher(projectId);
        return R.ok().put("name", name);
    }

    /**
     * 虚拟仿真列表
     *
     * @param vo
     * @param page
     * @param size
     * @return
     */
    @Override
    public PageInfo<ResultsEntity> list(ResultsVo vo, Integer page, Integer size) {

        //使用分页插件,核心代码就这一行
        PageHelper.startPage(page, size);
        List<ResultsEntity> list = achievementManagementDao.list(vo);
        PageInfo info = new PageInfo(list);

        for (ResultsEntity entity : list) {
            Integer number = achievementManagementDao.getImitationAchievement(entity.getProjectId()).size();
            entity.setNumber(number);
        }
/*
        Date begin = new Date();
        list.parallelStream().forEach(x -> {
            x.setNumber(achievementManagementDao.getImitationAchievement(x.getProjectId()).size());
            x.setProjectHiddenEntity(projectHiddenDao.selectByProjectId(x.getProjectId()));
        });
        Date end = new Date();
        // System.out.println("时间1：" + (end.getTime() - begin.getTime()));

        Date begin1 = new Date();
        for (ResultsEntity entity : list) {
            Integer number = achievementManagementDao.getImitationAchievement(entity.getProjectId()).size();
            entity.setNumber(number);

            ProjectHiddenEntity hiddenEntity = projectHiddenDao.selectByProjectId(entity.getProjectId());
            entity.setProjectHiddenEntity(hiddenEntity);
        }
        Date end1 = new Date();*/
        //System.out.println("时间2：" + (end1.getTime() - begin1.getTime()));


        // PageInfo<ResultsEntity> ret = new PageInfo<>();

        return info;
    }

    @Override
    public List<ReportEntity> reportList(ResultsVo vo, Integer page, Integer size) {
        List<ReportEntity> reportList = achievementManagementDao.reportList(vo);
        return reportList;
    }

    @Override
    public PageInfo<ResultsEntity> newList(ResultsVo vo, Integer page, Integer size) {
        //使用分页插件,核心代码就这一行
        PageHelper.startPage(page, size);
        List<ResultsEntity> newList = achievementManagementDao.newList(vo);
        for (ResultsEntity entity : newList) {
            Integer number = achievementManagementDao.getTeachAchievement(entity.getId()).size();
            entity.setNumber(number);
        }

        PageInfo info = new PageInfo(newList);
        return info;
    }


    @Override
    @Transactional
    public R addReport(ExperimentalReportEntity entity) {
        //        RecordVo record = entity.getEntity();
        //        //1.信息校验
        //        String userName = entity.getUserName();
        //        Integer userid = record.getUserid();
        //        String teacherName = entity.getTeacherName();
        //        //获取学生id和用户id
        //        StudentEntity student = userInfoDao.queryStudentId(userName,userid);
        //        //获取学生姓名
        //        List<String> studentName = userInfoDao.queryStudentName(userid);
        //        if (studentName.size()==0){
        //            return R.error("不存在的学生姓名");
        //        }else if (student==null){
        //            log.info("学生姓名和用户id不匹配！！！");
        //            return R.error("错误的个人信息！");
        //        }

        //2.根据实验得分 自动获取评语信息

        //3.将评语对应的commentId放入entity以便一齐存入实验报告表
        //（无判分系统暂写score=100）
        //        record.setScore(100);
        //        //根据成绩取出对应的评语和评语id
        //        Integer score = record.getScore();
        //        if (score!=null) {
        //            if (score<60){
        //                entity.setCommentId(1);//成绩不合格
        //            }else if (score>=60&&score<70){
        //                entity.setCommentId(2);//合格
        //            }else if (score>=70&&score<90){
        //                entity.setCommentId(3);//良好
        //            }else {
        //                entity.setCommentId(4);//优秀
        //            }
        //        }else {
        //            return R.error("系统异常，成绩为空！");
        //        }
        //4.添加实验记录表信息

        //        projectRecordDao.addReport(record);//保存至tms_project_record并返回主键
        //5.添加实验报告表信息
        //        entity.setStudentId(student.getStudentId()).setEventId(record.getEventId());
        achievementManagementDao.addReport(entity);

        return R.ok();
    }

    @Override
    public R queryReport(Integer reportId) {
        HashMap<String, Object> map = new HashMap<>();
        AchievementManagementVO report = achievementManagementDao.queryReport(reportId);
        Integer eventId = report.getEventId();
        ProjectRecordEntity record = projectRecordDao.queryReport(eventId);
        String userName = userInfoDao.queryUserReport(record.getUserid());
        if (report.getProjectId() != null) {
            String evaluationName = achievementManagementDao.queryEvaluationName(report.getProjectId());
            report.setEvaluationName(evaluationName);
        }
        //python实验数据
        Integer systemId = report.getSystemId();
        if (systemId == 1 | systemId == 4 | systemId == 5) {
            List<AchievementManagementVO> data = achievementManagementDao.queryPythonTest(eventId);
            map.put("data", data);
        }
        report.setUserName(userName);
        map.put("report", report);
        map.put("record", record);
        return R.ok().put("data", map);
    }

    @Override
    @Transactional
    public R deleteReport(List<Integer> reportId,List<Integer> recordId) {
        for (int i = 0; i < reportId.size(); i++) {
            achievementManagementDao.deleteReport(reportId.get(i), recordId.get(i));
        }
        return R.ok();
    }

    @Override
    public R queryEvaluationReport(String searchContant, Integer projectId, Integer page, Integer size) {
        Page<T> tPage = new Page<>(page, size);
        IPage<AchievementManagementVO> data = achievementManagementDao.queryEvaluationReport(tPage, searchContant, projectId);


        PageUtils result = new PageUtils(data);

        return R.ok().put("data", result);
    }

    @Override
    public void exportAchievement(HttpServletResponse response, String ids, Integer source) throws Exception {

        List<Integer> list = new ArrayList<>();

        String[] split = ids.split(",");
        for (int i = 0; i < split.length; i++) {
            list.add(Integer.parseInt(split[i]));
        }
        //source 1.练习 2考核
        switch (source) {
            case 1:
                //1.获取数据
                List<AchievementImportRequest> achievement = achievementManagementDao.queryAchievement(list);
                //2.加载模板流数据
                ClassPathResource resource = new ClassPathResource("excel-template/川大成绩管理实验记录导出模板 (1).xlsx");
                InputStream inputStream = resource.getInputStream();
                //3、通过工具类下载文件
                new ExcelExportUtil(AchievementImportRequest.class, Constant.ROW_INDEX, Constant.STYLE_INDEX).
                        export(response, inputStream, achievement, "班级实验成绩.xlsx");
                break;
            case 2:
                //1.获取数据

                /*List<ImportAssessmentRequest> assessment = null;
                for (Integer id : list) {
                    assessment = achievementManagementDao.queryAchievementByAssessment(id);
                }*/
                //Integer id = Integer.valueOf(list.get(0));
                List<ImportAssessmentRequest> assessment = achievementManagementDao.queryAchievementByAssessment(list);

                //2.加载模板流数据
                ClassPathResource resource1 = new ClassPathResource("excel-template/川大成绩管理实验记录导出模板 (2).xlsx");
                InputStream inputStream1 = resource1.getInputStream();

                //3、通过工具类下载文件
                new ExcelExportUtil(ImportAssessmentRequest.class, Constant.ROW_INDEX, Constant.STYLE_INDEX).
                        export(response, inputStream1, assessment, "班级实验成绩.xlsx");
                break;
            default:
                return;
        }

    }

    @Override
    public R queryAchievement(Integer systemId, Integer projectPermissions, String searchContant, String startingtime,
                              String endtime, Integer month) {
        try {
            List<AchievementManagementVO> fictitious;
            List<AchievementManagementVO> education;
            List<AchievementManagementVO> vo = new ArrayList<>();
            List<Integer> educationResult = achievementManagementDao.countEducationProjectId();//获取教学实验的主键id
            List<Integer> fictitiousResult = achievementManagementDao.countFictitiousProjectId();//获取教学实验的主键id
            if (projectPermissions == null) {
                projectPermissions = -9999;
            }
            if (projectPermissions == 0) {//只查询练习项目
                fictitious = achievementManagementDao.queryFictitiousRecord(systemId, searchContant, startingtime, endtime, month, fictitiousResult);
                vo = fictitious;
            } else if (projectPermissions == 1) {//只查询考核项目
                education = achievementManagementDao.queryEducationRecord(systemId, searchContant, startingtime, endtime, month, educationResult);
                vo = education;
            } else {//查询练习+考核项目
                if (educationResult.size() == 0) {
                    if (fictitiousResult.size() > 0) {
                        fictitious = achievementManagementDao.queryFictitiousRecord(systemId, searchContant, startingtime, endtime, month, fictitiousResult);
                        vo = fictitious;
                    }
                } else {
                    education = achievementManagementDao.queryEducationRecord(systemId, searchContant, startingtime, endtime, month, educationResult);
                    vo = education;
                    if (fictitiousResult.size() > 0) {
                        fictitious = achievementManagementDao.queryFictitiousRecord(systemId, searchContant, startingtime, endtime, month, fictitiousResult);
                        for (int i = 0; i < fictitious.size(); i++) {
                            vo.add(fictitious.get(i));
                        }
                        //按创建时间时间排序
                        Collections.sort(vo, new Comparator<AchievementManagementVO>() {
                            @Override
                            public int compare(AchievementManagementVO o1, AchievementManagementVO o2) {
                                int flag = o2.getCreationTime().compareTo(o1.getCreationTime());
                                return flag;
                            }
                        });
                    }
                }
            }
            return R.ok().put("data", vo);
        } catch (RuntimeException e) {
            log.info(e.getMessage());
            return R.error(500, "系统异常！！！");
        }
    }

    @Override
    @Transactional
    public R updateComment(ExperimentalReportEntity entity) {

        achievementManagementDao.updateComment(entity);
        return R.ok();
    }

    @Override
    @Transactional
    public void updateSignatrue(ExperimentalReportEntity entity) {
        Integer isSignature = entity.getIsSignature();
        Integer signatureId = entity.getSignatureId();
        Integer reportId = entity.getReportId();
        String comment = entity.getComment();
        if (comment == "") {
            comment = null;
        }
        if (isSignature == 1) {
            achievementManagementDao.updateSignatrue(1, signatureId, reportId, comment);
        } else {
            achievementManagementDao.updateSignatrue(0, null, reportId, comment);
        }
    }


    @Override
    public R deleteReportById(List<Integer> projectIds, Integer projectPermissions, List<Integer> ids) {

        ProjectHiddenEntity projectHiddenEntity = new ProjectHiddenEntity();
        //projectHiddenEntity.setId(null);
        int num = 0;

        if (projectPermissions == 1) {//实验教学
            for (Integer id : ids) {
                List<AchievementManagementVO> teachAchievements = achievementManagementDao.getTeachAchievement(id);
                for (AchievementManagementVO teachAchievement : teachAchievements) {
                    Integer reportId = teachAchievement.getReportId();
                    Integer recordId = teachAchievement.getRecordId();
                    achievementManagementDao.deleteReport(reportId, recordId);
                }

                //将教学实验隐藏
                projectHiddenEntity.setTeachId(id);
                projectHiddenEntity.setIsHidden(1);
                num = projectHiddenDao.insert(projectHiddenEntity);
                if (num <= 0) {
                    R.error(400, "删除失败！");
                }

            }
        } else {
            for (Integer projectId : projectIds) {//虚仿实验
                List<AchievementManagementVO> imitationAchievements = achievementManagementDao.getImitationAchievement(projectId);
                for (AchievementManagementVO imitationAchievement : imitationAchievements) {
                    Integer reportId = imitationAchievement.getReportId();
                    Integer recordId = imitationAchievement.getRecordId();
                    achievementManagementDao.deleteReport(reportId, recordId);
                }

                //将虚仿项目隐藏

                projectHiddenEntity.setProjectId(projectId);
                projectHiddenEntity.setIsHidden(1);
                num = projectHiddenDao.insert(projectHiddenEntity);
                if (num <= 0) {
                    R.error(400, "删除失败！");
                }

            }
        }


        return R.ok();

    }

    @Override
    public List<AchievementManagementVO> getImitationAchievement(Integer projectId) {
        List<AchievementManagementVO> list = achievementManagementDao.getImitationAchievement(projectId);


        return list;
    }

    @Override
    public List<AchievementManagementVO> getTeachAchievement(Integer id) {
        return achievementManagementDao.getTeachAchievement(id);
    }

    @Override
    public Integer modify(ExperimentalReportEntity entity) {
        return achievementManagementDao.modify(entity);
    }

}
