package com.msdw.tms.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.msdw.tms.common.exception.ExceptionCast;
import com.msdw.tms.common.utils.Constant;
import com.msdw.tms.dao.EvaluationQuestionDao;
import com.msdw.tms.dao.EvaluationRecordDao;
import com.msdw.tms.entity.EvaluationQuestionEntity;
import com.msdw.tms.entity.EvaluationRecordEntity;
import com.msdw.tms.entity.response.CommonCode;
import com.msdw.tms.entity.vo.*;
import com.msdw.tms.service.EvaluationQuestionService;
import com.msdw.tms.service.EvaluationRecordService;
import com.msdw.tms.service.QuestionsService;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.stream.Collectors;


@Service("evaluationRecordService")
public class EvaluationRecordServiceImpl extends ServiceImpl<EvaluationRecordDao, EvaluationRecordEntity> implements EvaluationRecordService {

    @Resource
    QuestionsService questionsService;

    @Resource
    EvaluationQuestionService evaluationQuestionService;

    @Resource
    EvaluationQuestionDao evaluationQuestionDao;

    @Autowired
    StringRedisTemplate redisTemplate;

    @Autowired
    ThreadPoolExecutor executor;

    private String REMAINING_TINE_KEY = "REMAINING_TINE";
    @Override
    @Transactional
    public TMSEvaluationRecordVO startEvaluation(Integer userId) {
        //TODO 接口幂等性处理
        //根据用户id和是否提交，查询测评记录信息
        EvaluationRecordEntity one = this.getOne(new QueryWrapper<EvaluationRecordEntity>().eq("user_id", userId)
                .eq("evaluation_status", Constant.Submit.NOT_SUBMIT.getType()));

        if (one == null) {
            //随机抽取试题数据
            EvaluationVO evaluation = questionsService.randomQuestions();

            //保存测评记录
            EvaluationRecordEntity evaluationRecord = new EvaluationRecordEntity();
            evaluationRecord.setUserId(userId);
            evaluationRecord.setEvaluationStatus(Constant.Submit.NOT_SUBMIT.getType());
            evaluationRecord.setTotalQuestionNum(evaluation.getQuestionNum());
            evaluationRecord.setCurrentQuestionSortNo(1);
            evaluationRecord.setStartTime(new Date());
            this.save(evaluationRecord);

            List<EvaluationQuestionEntity> evaluationQuestionList = evaluation.getQuestions().stream().map(question -> {
                EvaluationQuestionEntity evaluationQuestionEntity = new EvaluationQuestionEntity();
                evaluationQuestionEntity.setEvaluationRecordId(evaluationRecord.getId());
                evaluationQuestionEntity.setQuestionId(question);
                return evaluationQuestionEntity;
            }).collect(Collectors.toList());

            List<Integer> list = averageValue(evaluation.getQuestionNum());
            for (int i = 0; i < list.size(); i++) {
                evaluationQuestionList.get(i).setQuestionSortNo(i + 1);
                evaluationQuestionList.get(i).setQuestionPoints(list.get(i));
            }
            evaluationQuestionService.saveBatch(evaluationQuestionList);

            one = evaluationRecord;
        }

        TMSEvaluationRecordVO evaluationRecordVO = this.getBaseMapper()
                .selectVOByInfo(one.getId(), one.getCurrentQuestionSortNo());
        evaluationRecordVO.setQuestionTypeName(getQuestionTypeName(evaluationRecordVO.getQuestionType()));
        return evaluationRecordVO;
    }

    private String getQuestionTypeName(Integer questionType) {
        String questionTypeName = "";
        //处理试题类型
        if (questionType.equals(Constant.QuestionType.SINGLE_CHOICE.getType())) {
            //单选题
            questionTypeName = Constant.QuestionType.SINGLE_CHOICE.getDesc();
        } else if (questionType.equals(Constant.QuestionType.MULTIPLE_CHOICE.getType())) {
            //多选题
            questionTypeName = Constant.QuestionType.MULTIPLE_CHOICE.getDesc();
        } else if (questionType.equals(Constant.QuestionType.TRUE_OR_FALSE.getType())) {
            //判断题
            questionTypeName = Constant.QuestionType.TRUE_OR_FALSE.getDesc();
        } else {
            ExceptionCast.cast(CommonCode.QUESTIONTYPE_INVALID);
        }
        return questionTypeName;
    }

    /**
     * 平均分配分值
     *
     * @param number 判分点数量
     * @return
     */
    private List<Integer> averageValue(Integer number) {
        if (number == null || number == 0) {
            ExceptionCast.cast(CommonCode.EVALUATION_QUESTION_NUM_INVALID);
        }
        Integer score = 100 / number;
        List<Integer> list = new ArrayList<>();
        if (100 % number == 0) {
            for (int i = 0; i < number; i++) {
                list.add(score);
            }
        } else {
            int n;
            for (int i = 0; i < number; i++) {
                list.add(score);
            }
            for (int i = 100 % number; i > 0; i--) {
                n = score + 1;
                list.set(i, n);
            }
        }
        return list;
    }

    @Override
    @Transactional
    public TMSEvaluationRecordVO convertQuestion(Integer evaluationRecordId,
                                              Integer currentQuestionSortNo,
                                              String userAnswer,
                                              Integer changeSortNo) throws ExecutionException, InterruptedException {
        if (currentQuestionSortNo < 1
                || currentQuestionSortNo > this.getById(evaluationRecordId).getTotalQuestionNum()
                || changeSortNo < 1
                || changeSortNo > this.getById(evaluationRecordId).getTotalQuestionNum()) {
            ExceptionCast.cast(CommonCode.INVALID_PARAM);
        }
        //提交当前试题信息
        submitCurrentQuestion(evaluationRecordId, currentQuestionSortNo, userAnswer);
//        CompletableFuture<Void> submitCurrentQuestionFuture = CompletableFuture.runAsync(()
//                -> submitCurrentQuestion(evaluationRecordId, currentQuestionSortNo, userAnswer), executor);

        //修改测评记录表当前题目顺序号
        updateCurrentSortNo(changeSortNo, evaluationRecordId);
//        CompletableFuture<Void> updateCurrentSortNoFuture = CompletableFuture.runAsync(()
//                -> updateCurrentSortNo(changeSortNo, evaluationRecordId), executor);

//        CompletableFuture<EvaluationRecordVO> future = CompletableFuture.supplyAsync(()
//                -> this.getBaseMapper().selectVOByInfo(evaluationRecordId, changeSortNo), executor);

        TMSEvaluationRecordVO evaluationRecordVO = this.getBaseMapper().selectVOByInfo(evaluationRecordId, changeSortNo);

//        CompletableFuture.allOf(submitCurrentQuestionFuture, updateCurrentSortNoFuture, future).get();

//        EvaluationRecordVO evaluationRecordVO = future.get();
        //查询下一题信息
        evaluationRecordVO.setQuestionTypeName(getQuestionTypeName(evaluationRecordVO.getQuestionType()));
        return evaluationRecordVO;
    }

    @Override
    public EvaluationRecordSubmitVO submitEvaluation(Integer evaluationRecordId,
                                                     Integer currentQuestionSortNo,
                                                     String userAnswer,
                                                     Integer userId) throws ExecutionException, InterruptedException {
        //清除redis中的倒计时
        CompletableFuture<Void> deleteKeyFuture = CompletableFuture.runAsync(() -> {
            String key = REMAINING_TINE_KEY + userId;
            redisTemplate.delete(key);
        }, executor);

//        CompletableFuture<Void> submitCurrentQuestionFuture = CompletableFuture.runAsync(()
//                -> submitCurrentQuestion(evaluationRecordId, currentQuestionSortNo, userAnswer), executor);
//      2020.09.30  清除上面异步操作，需先提交并统计最后一题的结果
        submitCurrentQuestion(evaluationRecordId, currentQuestionSortNo, userAnswer);

        //修改提交时间和测评记录状态
        CompletableFuture<Void> updateEvaluationFuture = CompletableFuture.runAsync(() -> {
            EvaluationRecordEntity entity = new EvaluationRecordEntity();
            entity.setId(evaluationRecordId);
            entity.setEvaluationStatus(Constant.Submit.SUBMIT.getType());
            entity.setSubmitTime(new Date());
            this.updateById(entity);
        }, executor);

        EvaluationRecordSubmitVO submitVO = new EvaluationRecordSubmitVO();

        //查询总成绩
        CompletableFuture<Void> sumScoreFuture = CompletableFuture.runAsync(() -> {
            int sumScore = evaluationQuestionDao.sumScore(evaluationRecordId);
            //得分
            submitVO.setTotalScore(sumScore);
            //是否通过
//            submitVO.setIsPassed(sumScore > Constant.PASSING_SCORE ? Constant.EVALUATIONN_PASSED : Constant.EVALUATIONN_NOT_PASSED);
            //成绩大于等于60分通过
            submitVO.setIsPassed(sumScore >= Constant.PASSING_SCORE ? Constant.EVALUATIONN_PASSED : Constant.EVALUATIONN_NOT_PASSED);
        }, executor);

        CompletableFuture<EvaluationRecordEntity> getByIdFuture = CompletableFuture.supplyAsync(() -> {
            EvaluationRecordEntity evaluationRecord = this.getById(evaluationRecordId);
            BeanUtils.copyProperties(evaluationRecord, submitVO);
            return evaluationRecord;
        }, executor);

        CompletableFuture<Void> countFuture = getByIdFuture.thenAcceptAsync((res) -> {
            QueryWrapper<EvaluationQuestionEntity> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("evaluation_record_id", evaluationRecordId)
                    .eq("is_ture", Constant.QuestionIsTure.TRUE.getType());
            //查询答对几题
            int count = evaluationQuestionService.count(queryWrapper);
            //答对几道
            submitVO.setCorrectQuestionNum(count);
            //正确率
            // 创建一个数值格式化对象
            NumberFormat numberFormat = NumberFormat.getInstance();
            // 设置精确到小数点后2位
            numberFormat.setMaximumFractionDigits(2);
            submitVO.setCorrectRate(numberFormat.format((float) count / (float) res.getTotalQuestionNum() * 100) + "%");

        }, executor);
//去除不需要的数据
//        CompletableFuture.allOf(deleteKeyFuture, submitCurrentQuestionFuture, updateEvaluationFuture,
        CompletableFuture.allOf(deleteKeyFuture, updateEvaluationFuture,
                sumScoreFuture, getByIdFuture, countFuture).get();

        return submitVO;
    }

    @Override
    public EvaluationRecordDetailVO evaluationDetail(Integer evaluationRecordId) throws ExecutionException, InterruptedException {
        EvaluationRecordDetailVO evaluationRecordDetailVO = new EvaluationRecordDetailVO();

        CompletableFuture<Void> sumScoreFuture = CompletableFuture.runAsync(() -> {
            //查询总成绩
            int sumScore = evaluationQuestionDao.sumScore(evaluationRecordId);
            evaluationRecordDetailVO.setTotalScore(sumScore);
        }, executor);

        CompletableFuture<Void> evaluationDetailVOSFuture = CompletableFuture.runAsync(() -> {
            List<EvaluationDetailVO> evaluationDetailVOS = evaluationQuestionDao.selectDetailByInfo(evaluationRecordId);
            evaluationRecordDetailVO.setEvaluationDetailVOS(evaluationDetailVOS);
        }, executor);

        CompletableFuture.allOf(sumScoreFuture, evaluationDetailVOSFuture).get();
        return evaluationRecordDetailVO;
    }

    @Override
    public String selectNotMade(Integer evaluationRecordId) {

        QueryWrapper<EvaluationQuestionEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("evaluation_record_id", evaluationRecordId)
                .eq("question_status", Constant.QuestionStatus.NOT_MADE.getType());

        List<EvaluationQuestionEntity> list = evaluationQuestionService.list(queryWrapper);
        List<Integer> collect = list.stream().map(EvaluationQuestionEntity::getQuestionSortNo).collect(Collectors.toList());
        if (collect.size() == 0) {
            return "已完成全部题目";
        }
        return "还有以下题目未完成" + collect.toString();
    }

    @Override
    public void cleanRemainingTime(Integer userId) {
        //清除redis中的倒计时
        String key = REMAINING_TINE_KEY + userId;
        redisTemplate.delete(key);
    }

    @Override
    public boolean isOpenExperiment(Integer userId) {
        List<Integer> list = evaluationQuestionDao.sumScoreListByUserId(userId);
        if (list != null && list.size() > 0) {
            for (Integer integer : list) {
                if (integer > Constant.PASSING_SCORE) {
                    return true;
                }
            }
        }
        return false;
    }

//    @Override
//    public boolean hasNotMadeEvaluation(Integer userId) {
//        //先判断数据库是否已经有了此次测评
//        //如果有，继续，如果没有，重新开始
//        ValueOperations<String, String> ops = redisTemplate.opsForValue();
//        String key = REMAINING_TINE_KEY + userId;
//        if (StringUtils.isNotEmpty(ops.get(key))) {
//            //继续上次
//
//        }
//
//        return false;
//    }

    //提交当前试题，并修改当前测评记录表
    private void submitCurrentQuestion(Integer evaluationRecordId,
                                       Integer currentQuestionSortNo,
                                       String userAnswer) {
        //1、根据当前传递的evaluationRecordId和currentQuestionSortNo查询当前试题
        EvaluationQuestionVO evaluationQuestionVO = evaluationQuestionDao.selectByInfo(evaluationRecordId, currentQuestionSortNo);

        if (evaluationQuestionVO == null) {
            ExceptionCast.cast(CommonCode.INVALID_PARAM);
        }
        // 题目分值
        Integer questionPoints = evaluationQuestionVO.getQuestionPoints();
        //正确答案
        String answer = evaluationQuestionVO.getAnswer();

        String userString = "";
        String answerString = "";
        char[] arr = userAnswer.toCharArray();
        Arrays.sort(arr);
        for (char a : arr){
            userString = userString + String.valueOf(a);
        }
        char[] array = answer.toCharArray();
        Arrays.sort(array);
        for (char b : array){
            answerString = answerString + String.valueOf(b);
        }
        //修改当前试题状态
        //记录用户的答案
        //判断当前试题正误
        //记录试题得分
        EvaluationQuestionEntity evaluationQuestion = new EvaluationQuestionEntity();
        evaluationQuestion.setId(evaluationQuestionVO.getId());
        evaluationQuestion.setUserAnswer(userAnswer);
        //如果用户答案为空，则设置状态为未作，设为错，得分设为0
//        if (StringUtils.isEmpty(userAnswer)) {
        if (StringUtils.isEmpty(userString)) {
            evaluationQuestion.setQuestionStatus(Constant.QuestionStatus.NOT_MADE.getType());
            evaluationQuestion.setIsTure(Constant.QuestionIsTure.FALSE.getType());
            evaluationQuestion.setQuestionScore(0);
        } else {
            evaluationQuestion.setQuestionStatus(Constant.QuestionStatus.MADE.getType());
            if (!answerString.equals(userString)) {
                evaluationQuestion.setIsTure(Constant.QuestionIsTure.FALSE.getType());
                evaluationQuestion.setQuestionScore(0);
            } else {
                evaluationQuestion.setIsTure(Constant.QuestionIsTure.TRUE.getType());
                evaluationQuestion.setQuestionScore(questionPoints);
            }
        }
        evaluationQuestionDao.updateById(evaluationQuestion);

    }

    private void updateCurrentSortNo(Integer changeSortNo, Integer evaluationRecordId) {
        //修改测评记录表
        EvaluationRecordEntity evaluationRecordEntity = new EvaluationRecordEntity();
        evaluationRecordEntity.setCurrentQuestionSortNo(changeSortNo);
        evaluationRecordEntity.setId(evaluationRecordId);
        this.updateById(evaluationRecordEntity);
    }

//    @Override
//    @Transactional
//    public EvaluationRecordVO startEvaluation(Integer userId) {
//        //TODO 接口幂等性处理
//        //根据用户id和是否提交，查询测评记录信息
//        EvaluationRecordEntity one = this.getOne(new QueryWrapper<EvaluationRecordEntity>().eq("user_id", userId)
//                .eq("evaluation_status", Constant.Submit.NOT_SUBMIT.getType()));
//
//        if (one == null) {
//            //随机抽取试题数据
//            EvaluationVO evaluation = questionsService.randomQuestions();
//
//            //保存测评记录
//            EvaluationRecordEntity evaluationRecord = new EvaluationRecordEntity();
//            evaluationRecord.setUserId(userId);
//            evaluationRecord.setEvaluationStatus(Constant.Submit.NOT_SUBMIT.getType());
//            evaluationRecord.setTotalQuestionNum(evaluation.getQuestionNum());
//            evaluationRecord.setCurrentQuestionSortNo(1);
//            evaluationRecord.setStartTime(new Date());
//            this.save(evaluationRecord);
//
//            List<EvaluationQuestionEntity> evaluationQuestionList = evaluation.getQuestions().stream().map(question -> {
//                EvaluationQuestionEntity evaluationQuestionEntity = new EvaluationQuestionEntity();
//                evaluationQuestionEntity.setEvaluationRecordId(evaluationRecord.getId());
//                evaluationQuestionEntity.setQuestionId(question);
//                return evaluationQuestionEntity;
//            }).collect(Collectors.toList());
//
//            List<Integer> list = averageValue(evaluation.getQuestionNum());
//            for (int i = 0; i < list.size(); i++) {
//                evaluationQuestionList.get(i).setQuestionSortNo(i + 1);
//                evaluationQuestionList.get(i).setQuestionPoints(list.get(i));
//            }
//            evaluationQuestionService.saveBatch(evaluationQuestionList);
//
//            one = evaluationRecord;
//        }
//
//        EvaluationRecordVO evaluationRecordVO = this.getBaseMapper()
//                .selectVOByInfo(one.getId(), one.getCurrentQuestionSortNo());
//        evaluationRecordVO.setQuestionTypeName(getQuestionTypeName(evaluationRecordVO.getQuestionType()));
//        return evaluationRecordVO;
//    }
//
//    private String getQuestionTypeName(Integer questionType) {
//        String questionTypeName = "";
//        //处理试题类型
//        if (questionType.equals(Constant.QuestionType.SINGLE_CHOICE.getType())) {
//            //单选题
//            questionTypeName = Constant.QuestionType.SINGLE_CHOICE.getDesc();
//        } else if (questionType.equals(Constant.QuestionType.MULTIPLE_CHOICE.getType())) {
//            //多选题
//            questionTypeName = Constant.QuestionType.MULTIPLE_CHOICE.getDesc();
//        } else if (questionType.equals(Constant.QuestionType.TRUE_OR_FALSE.getType())) {
//            //判断题
//            questionTypeName = Constant.QuestionType.TRUE_OR_FALSE.getDesc();
//        } else {
//            ExceptionCast.cast(CommonCode.QUESTIONTYPE_INVALID);
//        }
//        return questionTypeName;
//    }
//
//    /**
//     * 平均分配分值
//     *
//     * @param number 判分点数量
//     * @return
//     */
//    private List<Integer> averageValue(Integer number) {
//        if (number == null || number == 0) {
//            ExceptionCast.cast(CommonCode.EVALUATION_QUESTION_NUM_INVALID);
//        }
//        Integer score = 100 / number;
//        List<Integer> list = new ArrayList<>();
//        if (100 % number == 0) {
//            for (int i = 0; i < number; i++) {
//                list.add(score);
//            }
//        } else {
//            int n;
//            for (int i = 0; i < number; i++) {
//                list.add(score);
//            }
//            for (int i = 100 % number; i > 0; i--) {
//                n = score + 1;
//                list.set(i, n);
//            }
//        }
//        return list;
//    }
//
//    @Override
//    @Transactional
//    public EvaluationRecordVO convertQuestion(Integer evaluationRecordId,
//                                              Integer currentQuestionSortNo,
//                                              String userAnswer,
//                                              Integer changeSortNo) throws ExecutionException, InterruptedException {
//        if (currentQuestionSortNo < 1
//                || currentQuestionSortNo > this.getById(evaluationRecordId).getTotalQuestionNum()
//                || changeSortNo < 1
//                || changeSortNo > this.getById(evaluationRecordId).getTotalQuestionNum()) {
//            ExceptionCast.cast(CommonCode.INVALID_PARAM);
//        }
//        //提交当前试题信息
//        submitCurrentQuestion(evaluationRecordId, currentQuestionSortNo, userAnswer);
////        CompletableFuture<Void> submitCurrentQuestionFuture = CompletableFuture.runAsync(()
////                -> submitCurrentQuestion(evaluationRecordId, currentQuestionSortNo, userAnswer), executor);
//
//        //修改测评记录表当前题目顺序号
//        updateCurrentSortNo(changeSortNo, evaluationRecordId);
////        CompletableFuture<Void> updateCurrentSortNoFuture = CompletableFuture.runAsync(()
////                -> updateCurrentSortNo(changeSortNo, evaluationRecordId), executor);
//
////        CompletableFuture<EvaluationRecordVO> future = CompletableFuture.supplyAsync(()
////                -> this.getBaseMapper().selectVOByInfo(evaluationRecordId, changeSortNo), executor);
//
//        EvaluationRecordVO evaluationRecordVO = this.getBaseMapper().selectVOByInfo(evaluationRecordId, changeSortNo);
//
////        CompletableFuture.allOf(submitCurrentQuestionFuture, updateCurrentSortNoFuture, future).get();
//
////        EvaluationRecordVO evaluationRecordVO = future.get();
//        //查询下一题信息
//        evaluationRecordVO.setQuestionTypeName(getQuestionTypeName(evaluationRecordVO.getQuestionType()));
//        return evaluationRecordVO;
//    }
//
//    @Override
//    public EvaluationRecordSubmitVO submitEvaluation(Integer evaluationRecordId,
//                                                     Integer currentQuestionSortNo,
//                                                     String userAnswer,
//                                                     Integer userId) throws ExecutionException, InterruptedException {
//        //清除redis中的倒计时
//        CompletableFuture<Void> deleteKeyFuture = CompletableFuture.runAsync(() -> {
//            String key = REMAINING_TINE_KEY + userId;
//            redisTemplate.delete(key);
//        }, executor);
//
//        CompletableFuture<Void> submitCurrentQuestionFuture = CompletableFuture.runAsync(()
//                -> submitCurrentQuestion(evaluationRecordId, currentQuestionSortNo, userAnswer), executor);
//
//        //修改提交时间和测评记录状态
//        CompletableFuture<Void> updateEvaluationFuture = CompletableFuture.runAsync(() -> {
//            EvaluationRecordEntity entity = new EvaluationRecordEntity();
//            entity.setId(evaluationRecordId);
//            entity.setEvaluationStatus(Constant.Submit.SUBMIT.getType());
//            entity.setSubmitTime(new Date());
//            this.updateById(entity);
//        }, executor);
//
//        EvaluationRecordSubmitVO submitVO = new EvaluationRecordSubmitVO();
//
//        //查询总成绩
//        CompletableFuture<Void> sumScoreFuture = CompletableFuture.runAsync(() -> {
//            int sumScore = evaluationQuestionDao.sumScore(evaluationRecordId);
//            //得分
//            submitVO.setTotalScore(sumScore);
//            //是否通过
//            submitVO.setIsPassed(sumScore > Constant.PASSING_SCORE ? Constant.EVALUATIONN_PASSED : Constant.EVALUATIONN_NOT_PASSED);
//        }, executor);
//
//        CompletableFuture<EvaluationRecordEntity> getByIdFuture = CompletableFuture.supplyAsync(() -> {
//            EvaluationRecordEntity evaluationRecord = this.getById(evaluationRecordId);
//            BeanUtils.copyProperties(evaluationRecord, submitVO);
//            return evaluationRecord;
//        }, executor);
//
//        CompletableFuture<Void> countFuture = getByIdFuture.thenAcceptAsync((res) -> {
//            QueryWrapper<EvaluationQuestionEntity> queryWrapper = new QueryWrapper<>();
//            queryWrapper.eq("evaluation_record_id", evaluationRecordId)
//                    .eq("is_ture", Constant.QuestionIsTure.TRUE.getType());
//            //查询答对几题
//            int count = evaluationQuestionService.count(queryWrapper);
//            //答对几道
//            submitVO.setCorrectQuestionNum(count);
//            //正确率
//            // 创建一个数值格式化对象
//            NumberFormat numberFormat = NumberFormat.getInstance();
//            // 设置精确到小数点后2位
//            numberFormat.setMaximumFractionDigits(2);
//            submitVO.setCorrectRate(numberFormat.format((float) count / (float) res.getTotalQuestionNum() * 100) + "%");
//
//        }, executor);
//
//        CompletableFuture.allOf(deleteKeyFuture, submitCurrentQuestionFuture, updateEvaluationFuture,
//                sumScoreFuture, getByIdFuture, countFuture).get();
//
//        return submitVO;
//    }
//
//    @Override
//    public EvaluationRecordDetailVO evaluationDetail(Integer evaluationRecordId) throws ExecutionException, InterruptedException {
//        EvaluationRecordDetailVO evaluationRecordDetailVO = new EvaluationRecordDetailVO();
//
//        CompletableFuture<Void> sumScoreFuture = CompletableFuture.runAsync(() -> {
//            //查询总成绩
//            int sumScore = evaluationQuestionDao.sumScore(evaluationRecordId);
//            evaluationRecordDetailVO.setTotalScore(sumScore);
//        }, executor);
//
//        CompletableFuture<Void> evaluationDetailVOSFuture = CompletableFuture.runAsync(() -> {
//            List<EvaluationDetailVO> evaluationDetailVOS = evaluationQuestionDao.selectDetailByInfo(evaluationRecordId);
//            evaluationRecordDetailVO.setEvaluationDetailVOS(evaluationDetailVOS);
//        }, executor);
//
//        CompletableFuture.allOf(sumScoreFuture, evaluationDetailVOSFuture).get();
//        return evaluationRecordDetailVO;
//    }
//
//    @Override
//    public String selectNotMade(Integer evaluationRecordId) {
//
//        QueryWrapper<EvaluationQuestionEntity> queryWrapper = new QueryWrapper<>();
//        queryWrapper.eq("evaluation_record_id", evaluationRecordId)
//                .eq("question_status", Constant.QuestionStatus.NOT_MADE.getType());
//
//        List<EvaluationQuestionEntity> list = evaluationQuestionService.list(queryWrapper);
//        List<Integer> collect = list.stream().map(EvaluationQuestionEntity::getQuestionSortNo).collect(Collectors.toList());
//        if (collect.size() == 0) {
//            return "已完成全部题目";
//        }
//        return "还有以下题目未完成" + collect.toString();
//    }
//
//    @Override
//    public void cleanRemainingTime(Integer userId) {
//        //清除redis中的倒计时
//        String key = REMAINING_TINE_KEY + userId;
//        redisTemplate.delete(key);
//    }
//
//    @Override
//    public boolean isOpenExperiment(Integer userId) {
//        List<Integer> list = evaluationQuestionDao.sumScoreListByUserId(userId);
//        if (list != null && list.size() > 0) {
//            for (Integer integer : list) {
//                if (integer > Constant.PASSING_SCORE) {
//                    return true;
//                }
//            }
//        }
//        return false;
//    }
//
////    @Override
////    public boolean hasNotMadeEvaluation(Integer userId) {
////        //先判断数据库是否已经有了此次测评
////        //如果有，继续，如果没有，重新开始
////        ValueOperations<String, String> ops = redisTemplate.opsForValue();
////        String key = REMAINING_TINE_KEY + userId;
////        if (StringUtils.isNotEmpty(ops.get(key))) {
////            //继续上次
////
////        }
////
////        return false;
////    }
//
//    //提交当前试题，并修改当前测评记录表
//    private void submitCurrentQuestion(Integer evaluationRecordId,
//                                       Integer currentQuestionSortNo,
//                                       String userAnswer) {
//        //1、根据当前传递的evaluationRecordId和currentQuestionSortNo查询当前试题
//        EvaluationQuestionVO evaluationQuestionVO = evaluationQuestionDao.selectByInfo(evaluationRecordId, currentQuestionSortNo);
//        if (evaluationQuestionVO == null) {
//            ExceptionCast.cast(CommonCode.INVALID_PARAM);
//        }
//        // 题目分值
//        Integer questionPoints = evaluationQuestionVO.getQuestionPoints();
//        //正确答案
//        String answer = evaluationQuestionVO.getAnswer();
//        //修改当前试题状态
//        //记录用户的答案
//        //判断当前试题正误
//        //记录试题得分
//        EvaluationQuestionEntity evaluationQuestion = new EvaluationQuestionEntity();
//        evaluationQuestion.setId(evaluationQuestionVO.getId());
//        evaluationQuestion.setUserAnswer(userAnswer);
//        //如果用户答案为空，则设置状态为未作，设为错，得分设为0
//        if (StringUtils.isEmpty(userAnswer)) {
//            evaluationQuestion.setQuestionStatus(Constant.QuestionStatus.NOT_MADE.getType());
//            evaluationQuestion.setIsTure(Constant.QuestionIsTure.FALSE.getType());
//            evaluationQuestion.setQuestionScore(0);
//        } else {
//            evaluationQuestion.setQuestionStatus(Constant.QuestionStatus.MADE.getType());
//            if (!answer.equals(userAnswer)) {
//                evaluationQuestion.setIsTure(Constant.QuestionIsTure.FALSE.getType());
//                evaluationQuestion.setQuestionScore(0);
//            } else {
//                evaluationQuestion.setIsTure(Constant.QuestionIsTure.TRUE.getType());
//                evaluationQuestion.setQuestionScore(questionPoints);
//            }
//        }
//        evaluationQuestionDao.updateById(evaluationQuestion);
//
//    }
//
//    private void updateCurrentSortNo(Integer changeSortNo, Integer evaluationRecordId) {
//        //修改测评记录表
//        EvaluationRecordEntity evaluationRecordEntity = new EvaluationRecordEntity();
//        evaluationRecordEntity.setCurrentQuestionSortNo(changeSortNo);
//        evaluationRecordEntity.setId(evaluationRecordId);
//        this.updateById(evaluationRecordEntity);
//    }
}