package com.msdw.tms.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.msdw.tms.common.exception.ExceptionCast;
import com.msdw.tms.common.utils.Constant;
import com.msdw.tms.dao.EvaluationRulesDao;
import com.msdw.tms.dao.QuestionsDao;
import com.msdw.tms.entity.EvaluationRulesEntity;
import com.msdw.tms.entity.QuestionsEntity;
import com.msdw.tms.entity.response.CommonCode;
import com.msdw.tms.entity.vo.EvaluationRulesVO;
import com.msdw.tms.service.EvaluationRulesService;
import com.msdw.tms.service.QuestionsService;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;


@Service("evaluationRulesService")
public class EvaluationRulesServiceImpl extends ServiceImpl<EvaluationRulesDao, EvaluationRulesEntity> implements EvaluationRulesService {

    @Resource
    QuestionsService questionsService;
    @Autowired
    private QuestionsDao questionsDao;

    //TODO 此表数据可存在redis中，提高系统性能
    @Override
    public EvaluationRulesVO getEvaluationRules() {
        EvaluationRulesVO evaluationRulesVO = new EvaluationRulesVO();

        EvaluationRulesEntity evaluationRules = this.getById(Constant.EVALUATION_RULES_ID);

        if (evaluationRules.getEvaluationType().equals(Constant.RulesType.CUSTOMIZE.getType())) {//自定义

            int totalNum = 0;
            if (evaluationRules.getIsSingleEnable().equals(Constant.IsEnable.ENABLE.getType())) {//启用
                totalNum += evaluationRules.getSingleNum();
            }
            if (evaluationRules.getIsMultipleEnable().equals(Constant.IsEnable.ENABLE.getType())) {//启用
                totalNum += evaluationRules.getMultipleNum();
            }
            if (evaluationRules.getIsJudgmentEnable().equals(Constant.IsEnable.ENABLE.getType())) {//启用
                totalNum += evaluationRules.getJudgmentNum();
            }

            //类型为自定义，测评试题数量是各个参与测评的试题类型数量之和
            evaluationRules.setQuestionNum(totalNum);
        }

        BeanUtils.copyProperties(evaluationRules, evaluationRulesVO);

        QueryWrapper<QuestionsEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_del", Constant.IsDel.NOT_DEL.getType()) //未删除
                .eq("is_enable", Constant.IsEnable.ENABLE.getType()); //启用

        evaluationRulesVO.setTotalQuestionNum(questionsService.count(queryWrapper));//总题数
        int totalSingleNum = questionsService.count(queryWrapper.eq("question_type",
                Constant.QuestionType.SINGLE_CHOICE.getType()));//单选题总数
//        int totalMultipleNum = questionsService.count(queryWrapper.eq("question_type",
//                Constant.QuestionType.MULTIPLE_CHOICE.getType()));//多选题总数
//        int totalJudgmentNum = questionsService.count(queryWrapper.eq("question_type",
//                Constant.QuestionType.TRUE_OR_FALSE.getType()));//判断题总数
//      2020.09.28
        int totalMultipleNum = questionsDao.countTotalMultipleNum();
        int totalJudgmentNum = questionsDao.countTotalJudgmentNum();

        evaluationRulesVO.setTotalSingleNum(totalSingleNum);
        evaluationRulesVO.setTotalMultipleNum(totalMultipleNum);
        evaluationRulesVO.setTotalJudgmentNum(totalJudgmentNum);

        return evaluationRulesVO;
    }

    @Override
    public boolean updateEvaluationRulesById(EvaluationRulesEntity evaluationRules) {
        //控制测评题数不能超过100道
        if (evaluationRules == null
                || evaluationRules.getQuestionNum() == null
                || evaluationRules.getQuestionNum() > 100
                || evaluationRules.getQuestionNum() < 0) {
            ExceptionCast.cast(CommonCode.QUESTION_NUM_INVALID);
        }

        QueryWrapper<QuestionsEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_del", Constant.IsDel.NOT_DEL.getType()) //未删除
                .eq("is_enable", Constant.IsEnable.ENABLE.getType()); //启用

        //判断题目是否超出范围
        //判断类型
        Integer evaluationType = evaluationRules.getEvaluationType();
        if (evaluationType.equals(Constant.RulesType.RANDOM.getType())) {//随机
            Integer questionNum = evaluationRules.getQuestionNum();

            //查询总题数
            int count = questionsService.count(queryWrapper);//总题数
            numIncalid(questionNum, count);

        } else if (evaluationType.equals(Constant.RulesType.CUSTOMIZE.getType())) {//自定义
            if (evaluationRules.getIsSingleEnable().equals(Constant.IsEnable.ENABLE.getType())) {//启用
                int singleNum = evaluationRules.getSingleNum();
                int totalSingleNum = questionsService.count(queryWrapper.eq("question_type",
                        Constant.QuestionType.SINGLE_CHOICE.getType()));//单选题总数
                numIncalid(singleNum, totalSingleNum);
            }
            if (evaluationRules.getIsMultipleEnable().equals(Constant.IsEnable.ENABLE.getType())) {//启用
                int multipleNum = evaluationRules.getMultipleNum();
//                int totalMultipleNum = questionsService.count(queryWrapper.eq("question_type",
//                        Constant.QuestionType.MULTIPLE_CHOICE.getType()));//多选题总数
                int totalMultipleNum = questionsDao.countTotalMultiplenum();
                numIncalid(multipleNum, totalMultipleNum);
            }
            if (evaluationRules.getIsJudgmentEnable().equals(Constant.IsEnable.ENABLE.getType())) {//启用
                int judgmentNum = evaluationRules.getJudgmentNum();
//                int totalJudgmentNum = questionsService.count(queryWrapper.eq("question_type",
//                        Constant.QuestionType.TRUE_OR_FALSE.getType()));//判断题总数
                int totalJudgmentNum = questionsDao.countTotalJudgmentnum();
                numIncalid(judgmentNum, totalJudgmentNum);
            }

        }

        return this.updateById(evaluationRules);
    }

    private void numIncalid(int num, int totalNum) {
        if (num < 0 || num > totalNum) {
            ExceptionCast.cast(CommonCode.QUESTION_NUM_INVALID);
        }
    }
}