package com.msdw.tms.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.msdw.tms.common.utils.Constant;
import com.msdw.tms.common.utils.PageUtils;
import com.msdw.tms.common.utils.R;
import com.msdw.tms.common.utils.poi.ExcelExportUtil;
import com.msdw.tms.dao.ProjectRecordDao;
import com.msdw.tms.dao.StudentDao;
import com.msdw.tms.entity.ExperimentalReportEntity;
import com.msdw.tms.entity.ProjectRecordEntity;
import com.msdw.tms.entity.request.ExperimentProjectRecordImportRequest;
import com.msdw.tms.entity.request.ProjectRecordImportRequest;
import com.msdw.tms.entity.resp.GetByUserRecordResp;
import com.msdw.tms.entity.vo.ProjectRecordExportVo;
import com.msdw.tms.entity.vo.ProjectRecordVo;
import com.msdw.tms.entity.vo.StudentVo;
import com.msdw.tms.service.ProjectRecordService;
import org.apache.poi.ss.formula.functions.T;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.io.InputStream;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.util.HashMap;
import java.util.List;
import java.util.stream.Collectors;


@Service("projectRecordService")
public class ProjectRecordServiceImpl extends ServiceImpl<ProjectRecordDao, ProjectRecordEntity> implements ProjectRecordService {

    @Autowired
    private ProjectRecordDao baseMapper;

    @Autowired
    private StudentDao studentDao;


    @Override
    public PageUtils getByKDUserRecord(Integer page, Integer size, Integer userId, Integer projectPermissions, Integer systemId) {

        ProjectRecordEntity projectRecord = new ProjectRecordEntity().setUserid(userId);

        //分页对象
        Page<T> page1 = new Page<>(page, size);

        IPage<ProjectRecordVo> projectRecordVos = this.baseMapper.getByKDUserRecord(page1, projectRecord, projectPermissions);

        PageUtils questionsPage = new PageUtils(projectRecordVos);

        return questionsPage;
    }

    @Override
    public PageUtils queryPage(Integer page, Integer size, Integer userId, Integer projectPermissions, Integer systemId) {

        /*GetByUserRecordResp projectRecord = new GetByUserRecordResp();
        projectRecord.setUserId(userId);*/

        //分页对象
        Page<T> page1 = new Page<>(page, size);

        IPage<GetByUserRecordResp> projectRecordVos = this.baseMapper.getByUserRecord(page1, userId, projectPermissions, systemId);

        PageUtils questionsPage = new PageUtils(projectRecordVos);

        return questionsPage;
    }

    @Override
    public PageUtils queryExperimentPage(Integer page, Integer size, Integer userid, Integer systemId) {

        ProjectRecordEntity projectRecord = new ProjectRecordEntity().setUserid(userid).setIsdel(Constant.IsDel.NOT_DEL.getType());

        //分页对象
        Page<T> page1 = new Page<>(page, size);

        IPage<ProjectRecordVo> projectRecordVos = this.baseMapper.getByUserExperimentRecord(page1, projectRecord, systemId);

        PageUtils questionsPage = new PageUtils(projectRecordVos);

        return questionsPage;
    }

    @Override
    public ProjectRecordVo queryUserScore(Integer userId) {

        Integer isdel = Constant.IsDel.NOT_DEL.getType();
        ProjectRecordVo projectRecordVo = this.baseMapper.getByUserScore(userId, isdel);

        //练习统计
        List<GetByUserRecordResp> sumByPracticeNum = this.baseMapper.sumByPracticeNum(userId);

        //练习总分
        Integer practiceByScore = 0;
        //练习总耗时
        Integer practiceByTime = 0;
        for (GetByUserRecordResp resp : sumByPracticeNum) {
            //累计总分
            practiceByScore += resp.getScore();
            //累计时间
            practiceByTime += resp.getTimeSum();
        }

        //考核统计
        List<ProjectRecordVo> sumByAssessmentNum = this.baseMapper.sumByAssessmentNum(userId);

        //练习总分
        Integer assessmenteByScore = 0;
        //练习总耗时
        double assessmentByTime = 0;
        for (ProjectRecordVo vo : sumByAssessmentNum) {
            //累计总分
            assessmenteByScore += vo.getScore();
            //累计时间
            assessmentByTime += vo.getTimeSum();
        }


        //设置实验次数
        Integer numer = sumByPracticeNum.size() + sumByAssessmentNum.size();
        projectRecordVo.setExperimentNumber(numer);


        //取平均分
        //BigDecimal getSumByScore = (new BigDecimal(practiceByScore).add(new BigDecimal(assessmenteByScore))).divide(new BigDecimal(numer)).setScale(2, RoundingMode.HALF_UP);

        if (practiceByScore + assessmenteByScore != 0) {
            double getSumByScore = (practiceByScore + assessmenteByScore) / numer;
            projectRecordVo.setAvgScore(getSumByScore);
        } else {
            projectRecordVo.setAvgScore(0);
        }


        //获得总耗时
        double getTime = practiceByTime + assessmentByTime;
        projectRecordVo.setTotalTime(new Double(getTime));


        Double totalTime = projectRecordVo.getTotalTime();
        if (totalTime != null) {
            DecimalFormat df = new DecimalFormat("0.00");
            double time = totalTime / 60;
            String format = df.format(time);
            projectRecordVo.setTotalTime(Double.parseDouble(format));
            //projectRecordVo.setTotalTime(projectRecordVo.getTotalTime());
        }


        return projectRecordVo;
    }

    @Override
    public ProjectRecordVo queryScore(Integer userId) {
        ProjectRecordVo projectRecordVo = this.baseMapper.queryScore(userId);
        return projectRecordVo;
    }

    @Override
    public void exportProjectRecord(HttpServletResponse response, Integer userId) throws Exception {
        //获取数据
        ProjectRecordEntity projectRecord = new ProjectRecordEntity().setUserid(userId).setIsdel(Constant.IsDel.NOT_DEL.getType());

        List<ProjectRecordExportVo> projectRecordVos = this.baseMapper.getByRecordImport(projectRecord);
//        List<ProjectRecordExportVo> projectRecordVos = this.baseMapper.getByUserRecordImport(projectRecord);

        StudentVo studentVo = studentDao.queryStudentDetails(userId);

        List<ProjectRecordImportRequest> importRequests = projectRecordVos.stream().map(project -> {
            ProjectRecordImportRequest request = new ProjectRecordImportRequest();
            BeanUtils.copyProperties(project, request);
//            request.setUserRecordstate(project.getRecordstate().equals(1) ? "已完成":"待评分");
            return request;
        }).collect(Collectors.toList());

        //2.加载模板流数据
//        org.springframework.core.io.Resource resource = new ClassPathResource("excel-template/虚拟仿真实验报告导出模板.xlsx");
//        FileInputStream fis = new FileInputStream(resource.getFile());
        ClassPathResource resource = new ClassPathResource("excel-template/练习记录模板.xlsx");
        InputStream inputStream = resource.getInputStream();


        //3、通过工具类下载文件
        new ExcelExportUtil(ProjectRecordImportRequest.class, Constant.ROW_INDEX_REPORT, Constant.STYLE_INDEX).
                exportExpand(response, inputStream, importRequests, studentVo,"练习记录.xlsx");
//        new ExcelExportUtil(ProjectRecordImportRequest.class, Constant.ROW_INDEX, Constant.STYLE_INDEX).
//                export(response, fis, importRequests, "实验报告导出.xlsx");
    }

    /**
     * 导出个人教学试验记录
     */
    @Override
    public void exportExperimentProjectRecord(HttpServletResponse response, Integer userId) throws Exception {
        //获取数据
        ProjectRecordEntity projectRecord = new ProjectRecordEntity().setUserid(userId).setIsdel(Constant.IsDel.NOT_DEL.getType());

        List<ProjectRecordExportVo> projectRecordVos = this.baseMapper.getByUserRecordImport(projectRecord);

        StudentVo studentVo = studentDao.queryStudentDetails(userId);

        List<ExperimentProjectRecordImportRequest> importRequests = projectRecordVos.stream().map(project -> {
            ExperimentProjectRecordImportRequest request = new ExperimentProjectRecordImportRequest();
            BeanUtils.copyProperties(project, request);
            return request;
        }).collect(Collectors.toList());

        //2.加载模板流数据
        ClassPathResource resource = new ClassPathResource("excel-template/考核记录模板.xlsx");
        InputStream inputStream = resource.getInputStream();


        //3、通过工具类下载文件
        new ExcelExportUtil(ExperimentProjectRecordImportRequest.class, Constant.ROW_INDEX_REPORT, Constant.STYLE_INDEX).
                exportExpand(response, inputStream, importRequests,studentVo, "考核记录.xlsx");
    }

    @Override
    public PageUtils querySchoolRecord(Integer page, Integer size, ProjectRecordVo projectRecord) {

        //分页对象
        Page<T> page1 = new Page<>(page, size);
        projectRecord.setIsdel(Constant.IsDel.NOT_DEL.getType());
        IPage<ProjectRecordVo> schoolRecord = this.baseMapper.getBySchoolRecord(page1, projectRecord);
        PageUtils pageUtils = new PageUtils(schoolRecord);
        return pageUtils;
    }

    @Override
    public boolean updateUserRecord(ProjectRecordEntity projectRecord) {
        boolean result = this.baseMapper.updateUserRecord(projectRecord);
        return result;
    }

    @Override
    public void updateScore(ProjectRecordEntity vo) {
        this.baseMapper.updateScore(vo);
    }


}