package com.msdw.tms.service.impl;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.msdw.tms.common.exception.ExceptionCast;
import com.msdw.tms.common.utils.*;
import com.msdw.tms.common.utils.poi.ExcelExportUtil;
import com.msdw.tms.common.utils.poi.ExcelImportUtil;
import com.msdw.tms.dao.QuestionsDao;
import com.msdw.tms.entity.EvaluationRulesEntity;
import com.msdw.tms.entity.QuestionsEntity;
import com.msdw.tms.entity.XlsxTemplateEntity;
import com.msdw.tms.entity.request.QuestionsAddRequest;
import com.msdw.tms.entity.request.QuestionsImportRequest;
import com.msdw.tms.entity.request.QuestionsQueryRequest;
import com.msdw.tms.entity.request.QuestionsUpdateRequest;
import com.msdw.tms.entity.response.CommonCode;
import com.msdw.tms.entity.vo.*;
import com.msdw.tms.service.AliyunOssService;
import com.msdw.tms.service.EvaluationRulesService;
import com.msdw.tms.service.QuestionsService;
import com.msdw.tms.service.XlsxTemplateService;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

@Service("questionsService")
public class QuestionsServiceImpl extends ServiceImpl<QuestionsDao, QuestionsEntity> implements QuestionsService {

    @Resource
    AliyunOssService ossService;

    @Autowired
    private EvaluationRulesService evaluationRulesService;

    @Autowired
    private QuestionsDao questionsDao;

    @Resource
    XlsxTemplateService xlsxTemplateService;

    @Resource
    StringRedisTemplate stringRedisTemplate;

    private String REMAINING_TINE_KEY = "REMAINING_TINE";
    private String FAILURE_IMPORT_KEY = "FAILURE_IMPORT";

    /**
     * 条件加分页查询，题干模糊查询，未删除，修改时间降序
     */
    @Override
    public PageUtils queryQuestionsPage(Integer page, Integer size, QuestionsQueryRequest request) {
        // 请求包装类
        QueryWrapper<QuestionsEntity> queryWrapper = new QueryWrapper<>();

        // 只查询未被删除的试题
        queryWrapper.eq("is_del", Constant.IsDel.NOT_DEL.getType());

        // 判断请求体是否为空
        if (request != null) {
            // 题干：判断题干是否为空，不为空则加入搜索条件
            if (request.getQuestionStem() != null) {
                queryWrapper.like("question_stem", request.getQuestionStem());
            }
            // 题型：判断题型是否为空，不为空则加入搜索条件
            if (request.getQuestionType() != null) {
                queryWrapper.eq("question_type", request.getQuestionType());
            }
            //扩展其他搜索条件...
        }

        // 修改时间降序
        queryWrapper.orderByDesc("modify_time");

        IPage<QuestionsEntity> questionsIPage = this.page(
                new Query<QuestionsEntity>().getPage(page, size),
                queryWrapper
        );
        List<QuestionsEntity> records = questionsIPage.getRecords();

        List<QuestionsListVO> questions = records.stream().map(question -> {
            QuestionsListVO questionsListVO = new QuestionsListVO();
            BeanUtils.copyProperties(question, questionsListVO);
            //处理试题类型
            String questionTypeName = getQuestionTypeName(question.getQuestionType());
            questionsListVO.setQuestionTypeName(questionTypeName);
            //处理时间格式
            questionsListVO.setModifyTime(handleTime(question.getModifyTime()));
            return questionsListVO;
        }).collect(Collectors.toList());
        PageUtils questionsPage = new PageUtils(questionsIPage);
        questionsPage.setList(questions);
        return questionsPage;
    }

    //处理时间格式
    private String handleTime(Date date) {
        Instant instant = date.toInstant();
        ZoneId zoneId = ZoneId.systemDefault();
        LocalDateTime localDateTime = instant.atZone(zoneId).toLocalDateTime();
        return localDateTime.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")); // 当前日期和时间
    }

    @Override
    public QuestionsDetailVO getQuestionDetailById(Integer id) {
        {
            QuestionsDetailVO questionsDetailVO = new QuestionsDetailVO();
            //查询试题信息
            QuestionsEntity questionsEntity = this.getById(id);
            BeanUtils.copyProperties(questionsEntity, questionsDetailVO);
            //处理试题类型
            String questionTypeName = getQuestionTypeName(questionsEntity.getQuestionType());
            questionsDetailVO.setQuestionTypeName(questionTypeName);

            //处理每个选项是否为被设置为答案
            String answer = questionsEntity.getAnswer();

            if (answer.contains(Constant.A)) {
                questionsDetailVO.setAIsTrue(true);
            }
            if (answer.contains(Constant.B)) {
                questionsDetailVO.setBIsTrue(true);
            }
            if (answer.contains(Constant.C)) {
                questionsDetailVO.setCIsTrue(true);
            }
            if (answer.contains(Constant.D)) {
                questionsDetailVO.setDIsTrue(true);
            }
            if (answer.contains(Constant.E)) {
                questionsDetailVO.setEIsTrue(true);
            }
            if (answer.contains(Constant.F)) {
                questionsDetailVO.setFIsTrue(true);
            }
            return questionsDetailVO;
        }
    }

    private String getQuestionTypeName(Integer questionType) {
        String questionTypeName = "";
        //处理试题类型
        if (questionType.equals(Constant.QuestionType.SINGLE_CHOICE.getType())) {
            //单选题
            questionTypeName = Constant.QuestionType.SINGLE_CHOICE.getDesc();
        } else if (questionType.equals(Constant.QuestionType.MULTIPLE_CHOICE.getType())) {
            //多选题
            questionTypeName = Constant.QuestionType.MULTIPLE_CHOICE.getDesc();
        } else if (questionType.equals(Constant.QuestionType.TRUE_OR_FALSE.getType())) {
            //判断题
            questionTypeName = Constant.QuestionType.TRUE_OR_FALSE.getDesc();
        } else {
            ExceptionCast.cast(CommonCode.QUESTIONTYPE_INVALID);
        }
        return questionTypeName;
    }

    /**
     * 保存试题
     * 1、检验题型是否正确
     * 2、检验题干和选项是否有重复
     * 3、设置创建时间和修改时间
     */
    @Override
    @Transactional
    public boolean saveQuestion(QuestionsAddRequest questions) {
        //试题对象为空或题干内容为空，抛出参数不合法异常
        if (questions == null
                || StringUtils.isEmpty(questions.getQuestionStem())
                || questions.getQuestionType() == null) {
            ExceptionCast.cast(CommonCode.INVALID_PARAM);
        }

        //校验题型是否正确
        checkType(questions.getQuestionType());

        // 校验重复，不仅要校验题干，还要校验选项
        QuestionsEntity questionsEntity = new QuestionsEntity();
        BeanUtils.copyProperties(questions, questionsEntity);

        //构造根据内容查询试题数量的Wrapper
        QueryWrapper<QuestionsEntity> queryWrapper = constructDistinctWrapper(questionsEntity);

        int count = this.count(queryWrapper);
        if (count > 0) {//说明已存在
            //抛出题目已存在异常
            ExceptionCast.cast(CommonCode.QUESTION_EXISTS);
        }
        // 将传入的对象内容拷贝到QuestionsEntity并返回
        questionsEntity.setCreateTime(new Date());
        questionsEntity.setModifyTime(new Date());
        //TODO 创建者和修改者。。。

        return this.save(questionsEntity);
    }

    @Override
    @Transactional
    public boolean updateQuestionById(QuestionsUpdateRequest questions) {
        if (questions == null
                || questions.getId() == null
                || StringUtils.isEmpty(questions.getQuestionStem())) {
            ExceptionCast.cast(CommonCode.INVALID_PARAM);
        }

        // 判断id是否存在
        if (this.getById(questions.getId()) == null) {
            ExceptionCast.cast(CommonCode.INVALID_PARAM);
        }

        //校验题型是否正确
        checkType(questions.getQuestionType());

        //校验题干和内容重复
        QuestionsEntity questionsEntity = new QuestionsEntity();
        BeanUtils.copyProperties(questions, questionsEntity);
        //构造根据内容查询试题数量的Wrapper
        QueryWrapper<QuestionsEntity> queryWrapper = constructDistinctWrapper(questionsEntity);

        queryWrapper.notIn("id", questions.getId());

        int count = this.count(queryWrapper);
        if (count > 0) {//说明除了本题之外题干已存在
            //抛出题目已存在异常
            ExceptionCast.cast(CommonCode.QUESTION_EXISTS);
        }

        questionsEntity.setModifyTime(new Date());
        //TODO 设置修改者。。。

        return this.updateById(questionsEntity);
    }

    private void checkType(int type) {
        boolean include = false;
        for (Constant.QuestionType value : Constant.QuestionType.values()) {
            if (value.getType().equals(type)) {
                include = true;
            }
        }
        if (!include) {//题型不在枚举类中，不正确
            ExceptionCast.cast(CommonCode.QUESTIONTYPE_INVALID);
        }
    }

    private QueryWrapper<QuestionsEntity> constructDistinctWrapper(QuestionsEntity questions) {
        QueryWrapper<QuestionsEntity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_del", Constant.IsDel.NOT_DEL.getType())
                .eq("question_stem", questions.getQuestionStem());
        if (StringUtils.isNotEmpty(questions.getOptionA())) {
            queryWrapper.eq("option_a", questions.getOptionA());
        }
        if (StringUtils.isNotEmpty(questions.getOptionB())) {
            queryWrapper.eq("option_b", questions.getOptionB());
        }
        if (StringUtils.isNotEmpty(questions.getOptionC())) {
            queryWrapper.eq("option_c", questions.getOptionC());
        }
        if (StringUtils.isNotEmpty(questions.getOptionD())) {
            queryWrapper.eq("option_d", questions.getOptionD());
        }
        if (StringUtils.isNotEmpty(questions.getOptionE())) {
            queryWrapper.eq("option_e", questions.getOptionE());
        }
        if (StringUtils.isNotEmpty(questions.getOptionF())) {
            queryWrapper.eq("option_f", questions.getOptionF());
        }
        return queryWrapper;
    }

//    @Override
//    @Transactional
//    public boolean isEnable(Integer id) {
//        QuestionsEntity questionsEntity = new QuestionsEntity();
//        questionsEntity.setId(id);
//        QuestionsEntity byId = this.getById(id);
//        if (byId.getIsEnable().equals(Constant.IsEnable.ENABLE.getType())) {
//            questionsEntity.setIsEnable(Constant.IsEnable.NOT_ENABLE.getType());
//        } else if (byId.getIsEnable().equals(Constant.IsEnable.NOT_ENABLE.getType())) {
//            questionsEntity.setIsEnable(Constant.IsEnable.ENABLE.getType());
//        } else {
//            ExceptionCast.cast(CommonCode.INVALID_PARAM);
//        }
//
//        return this.updateById(questionsEntity);
//    }

    @Override
    @Transactional
    public boolean deleteByIds(List<Integer> asList) {
        List<QuestionsEntity> collect = asList.stream().map(item -> {
            QuestionsEntity questionsEntity = new QuestionsEntity();
            questionsEntity.setId(item);
            questionsEntity.setIsDel(Constant.IsDel.DEL.getType());
            return questionsEntity;
        }).collect(Collectors.toList());

        return updateBatchById(collect);
    }

    @Override
    @Transactional
    public R isEnable(Integer id) {
        QuestionsEntity questionsEntity = new QuestionsEntity();
//        Integer id = rules.getQuestionId();
        questionsEntity.setId(id);
        QuestionsEntity byId = questionsDao.selectById(id);
        EvaluationRulesEntity questions= questionsDao.queryNumberOfQuestions();
        if (byId.getIsEnable().equals(Constant.IsEnable.ENABLE.getType())) {
//            Integer types = rules.getTypes();
            Integer types = byId.getQuestionType();
            EvaluationRulesEntity rule = questionsDao.queryNumberOfTatolQuestions();
            Integer num = questionsDao.queryQuestionNumber(types);
            int rulesNumber = 0;
            switch (types){
                case 1:
                    rulesNumber = questions.getSingleNum();
                    break;
                case 2:
                    rulesNumber = questions.getMultipleNum();
                    break;
                case 3:
                    rulesNumber = questions.getJudgmentNum();
                    break;
            }
            if (num<=rulesNumber){
                return R.error(400,"试题数量不能少于测评设置对应题的数量");
            }else{
                questionsEntity.setIsEnable(Constant.IsEnable.NOT_ENABLE.getType());
            }

        } else if (byId.getIsEnable().equals(Constant.IsEnable.NOT_ENABLE.getType())) {
            questionsEntity.setIsEnable(Constant.IsEnable.ENABLE.getType());
        } else {
            ExceptionCast.cast(CommonCode.INVALID_PARAM);
        }

        int i = questionsDao.updateById(questionsEntity);


        if (i==1){
            return R.ok();
        }else {
            return R.error();
        }
    }

    /**
     * 1、校验题干重复和题型不对的，
     * 2、从列表中将该题剔除并记录index和导入失败的原因，
     * 3、名称和选项内容都相同的要去重
     * 返回数据：token，成功几条，失败几条
     * 将失败数据和失败信息保存在redis中，使用list类型，加上过期时间，key是token
     */
    @Override
    @Transactional
    public Map<String, String> importQuestion(MultipartFile file) throws IOException {
        //准备map接收导入失败的数据信息
        Map<String, String> failureRecord = new HashMap<>();
        //准备list收集失败数据
        List<QuestionsImportFailureVO> failureVOs = new ArrayList<>();

        // 参数合法性校验，只能上传.xlsx后缀的文件
        if (StringUtils.isBlank(file.getOriginalFilename())
                || !file.getOriginalFilename().substring(file.getOriginalFilename().lastIndexOf(".")).equals(Constant.EXCEL_SUFFIX)) {
            ExceptionCast.cast(CommonCode.EXCEL_FILE_INVALID);
        }

        // 使用excel导入工具类获取试题数据列表
        List<QuestionsImportRequest> list = new ExcelImportUtil(QuestionsImportRequest.class)
                .readExcel(file.getInputStream(), Constant.ROW_INDEX, Constant.CELL_INDEX);

        if (list == null || list.size() == 0) {
            ExceptionCast.cast(CommonCode.EXCEL_INVALID);
        }

        //去掉重复数据，得到去重后的数据
        ArrayList<QuestionsImportRequest> distinctList = list.stream().collect(Collectors.collectingAndThen(// 去重收集
                Collectors.toCollection(() ->
                        new TreeSet<>(Comparator.comparing(QuestionsImportRequest::toStringForCompare))),
                ArrayList::new
        ));

        //与元数据做差，得到重复的数据
        List<QuestionsImportRequest> repeatFactor = removeRepeatFactor(distinctList, list);

        //记录到失败信息列表
        repeatFactor.forEach(question -> {
            QuestionsImportFailureVO failureVO = new QuestionsImportFailureVO();
            BeanUtils.copyProperties(question, failureVO);
            failureVO.setFailureMsg(CommonCode.REPEAT_INEXCEL.message());
            failureVOs.add(failureVO);
        });

        List<QuestionsEntity> questions = distinctList.stream().map(question -> {
            //校验题干和内容重复
            QuestionsEntity questionsEntity = new QuestionsEntity();
            //属性对拷
            BeanUtils.copyProperties(question, questionsEntity);
            String questionTypeName = question.getQuestionTypeName();
            //根据题型名称得到题型号
            if (questionTypeName.equals(Constant.QuestionType.SINGLE_CHOICE.getDesc())) {
                //单选题
                questionsEntity.setQuestionType(Constant.QuestionType.SINGLE_CHOICE.getType());
            } else if (questionTypeName.equals(Constant.QuestionType.MULTIPLE_CHOICE.getDesc())) {
                //多选题
                questionsEntity.setQuestionType(Constant.QuestionType.MULTIPLE_CHOICE.getType());
            } else if (questionTypeName.equals(Constant.QuestionType.TRUE_OR_FALSE.getDesc())) {
                //判断题
                questionsEntity.setQuestionType(Constant.QuestionType.TRUE_OR_FALSE.getType());
            } else {
                // 题型不正确
                QuestionsImportFailureVO failureVO = new QuestionsImportFailureVO();
                BeanUtils.copyProperties(question, failureVO);
                failureVO.setFailureMsg(CommonCode.QUESTIONTYPE_INVALID.message());
                failureVOs.add(failureVO);
                return null;
            }

            //构造根据内容查询试题数量的Wrapper
            QueryWrapper<QuestionsEntity> queryWrapper = constructDistinctWrapper(questionsEntity);

            int count = this.count(queryWrapper);

            if (count > 0) {//说明已存在
                QuestionsImportFailureVO failureVO = new QuestionsImportFailureVO();
                BeanUtils.copyProperties(question, failureVO);
                failureVO.setFailureMsg(CommonCode.QUESTION_EXISTS.message());
                failureVOs.add(failureVO);
                return null;
            }
            //TODO 创建者和修改者。。。
            questionsEntity.setCreateTime(new Date());
            questionsEntity.setModifyTime(new Date());
            return questionsEntity;
        }).filter(Objects::nonNull)// 过滤掉为null的对象
                .collect(Collectors.toList());

        String token = "";

        // 有导入失败的数据，才会存入redis
        if (failureVOs.size() > 0) {
            //生成token
            token = FAILURE_IMPORT_KEY + UUID.randomUUID().toString().replace("-", "");

            ValueOperations<String, String> ops = stringRedisTemplate.opsForValue();
            String failureVOJson = JSON.toJSONString(failureVOs);

            ops.set(token, failureVOJson, 30 * 60, TimeUnit.SECONDS);
        }

        // 批量保存
        this.saveBatch(questions);

        failureRecord.put("token", token);
        failureRecord.put("successNum", (list.size() - failureVOs.size()) + "");
        failureRecord.put("failureNum", failureVOs.size() + "");

        return failureRecord;
    }

    private <T> List<T> removeRepeatFactor(List<T> source, List<T> dest) {
        if (source != null && dest != null) {
            if (source.size() != 0 && dest.size() != 0) {
                Collection<T> A = new ArrayList<>(source);
                List<T> B = new ArrayList<>(dest);
                A.retainAll(B);
                if (A.size() != 0) {
                    B.removeAll(A);
                }
                return B;
            }
        }
        return dest;
    }

    /**
     * 上传模板文件，修改模板信息表数据
     *
     * @param file 上传的文件
     * @return 文件处理的结果集
     * @throws IOException io异常
     */
    @Override
    @Transactional
    public FilesResult uploadFiles(MultipartFile file) throws IOException {
        // 参数合法性校验，只能上传.xlsx后缀的文件
        if (StringUtils.isBlank(file.getOriginalFilename())
                || !file.getOriginalFilename().substring(file.getOriginalFilename().lastIndexOf(".")).equals(Constant.EXCEL_SUFFIX)) {
            ExceptionCast.cast(CommonCode.EXCEL_FILE_INVALID);
        }

        FilesResult filesResult = ossService.uploadFiles(file);

        XlsxTemplateEntity xlsxTemplateEntity = new XlsxTemplateEntity();
        BeanUtils.copyProperties(filesResult, xlsxTemplateEntity);
        xlsxTemplateEntity.setId(Constant.XLSX_TEMPLATE_ID);
        xlsxTemplateService.updateById(xlsxTemplateEntity);

        return filesResult;
    }

    @Override
    public void downloadFiles(HttpServletResponse response) throws IOException {
        XlsxTemplateEntity xlsxTemplate = xlsxTemplateService.getById(Constant.XLSX_TEMPLATE_ID);
        ossService.downloadFiles(response, xlsxTemplate.getFileName());
    }

    @Override
    public EvaluationVO randomQuestions() {
        // 查询测评规则类型
        EvaluationRulesVO evaluationRules = evaluationRulesService.getEvaluationRules();
        EvaluationVO evaluation = new EvaluationVO();
        BeanUtils.copyProperties(evaluationRules, evaluation);
        Set<Integer> set = new HashSet<>();
        //根据测评规则的类型不同
        Integer evaluationType = evaluationRules.getEvaluationType();
        if (evaluationType.equals(Constant.RulesType.RANDOM.getType())) {//随机
            // 类型为随机，直接用题目数量
            Integer questionNum = evaluationRules.getQuestionNum();
            if (questionNum == null || questionNum == 0) {
                ExceptionCast.cast(CommonCode.EVALUATION_QUESTION_NUM_INVALID);
            }
            // 查询题库所有未删除，未禁用的题目id
            List<Integer> ids = this.getBaseMapper().selectIds(null);
            set = getRandomList(questionNum, ids);
        } else if (evaluationType.equals(Constant.RulesType.CUSTOMIZE.getType())) {//自定义
            // 单选题数量
            int singleNum = 0;
            // 多选题数量
            int multipleNum = 0;
            // 判断题数量
            int judgmentNum = 0;
            //判断各条目的题型是否启用
            if (evaluationRules.getIsSingleEnable().equals(Constant.IsEnable.ENABLE.getType())) {
                singleNum = evaluationRules.getSingleNum();
                if (singleNum == 0) {
                    ExceptionCast.cast(CommonCode.EVALUATION_QUESTION_NUM_INVALID);
                }
                List<Integer> ids = this.getBaseMapper().selectIds(Constant.QuestionType.SINGLE_CHOICE.getType());
                set.addAll(getRandomList(singleNum, ids));
            }
            if (evaluationRules.getIsMultipleEnable().equals(Constant.IsEnable.ENABLE.getType())) {
                multipleNum = evaluationRules.getMultipleNum();
                if (multipleNum == 0) {
                    ExceptionCast.cast(CommonCode.EVALUATION_QUESTION_NUM_INVALID);
                }
                List<Integer> ids = this.getBaseMapper().selectIds(Constant.QuestionType.MULTIPLE_CHOICE.getType());
                set.addAll(getRandomList(multipleNum, ids));
            }
            if (evaluationRules.getIsJudgmentEnable().equals(Constant.IsEnable.ENABLE.getType())) {
                judgmentNum = evaluationRules.getJudgmentNum();
                if (judgmentNum == 0) {
                    ExceptionCast.cast(CommonCode.EVALUATION_QUESTION_NUM_INVALID);
                }
                List<Integer> ids = this.getBaseMapper().selectIds(Constant.QuestionType.TRUE_OR_FALSE.getType());
                set.addAll(getRandomList(judgmentNum, ids));
            }
            // 总题数等于各类题目数量之和
            evaluation.setQuestionNum(singleNum + multipleNum + judgmentNum);
        }
        evaluation.setQuestions(set);

        return evaluation;
    }

    private <T> Set<T> getRandomList(int len, List<T> list) {
        Set<T> set = new HashSet<>();
        Random random = new Random();
        int i;
        while (true) {
            i = random.nextInt(list.size());
            set.add(list.get(i));
            if (set.size() >= len) {
                break;
            }
        }
        return set;
    }

    @Override
    public void exportQuestion(HttpServletResponse response) throws Exception {
        // 请求包装类
        QueryWrapper<QuestionsEntity> queryWrapper = new QueryWrapper<>();
        // 只查询未被删除且启用的试题
        queryWrapper.eq("is_del", Constant.IsDel.NOT_DEL.getType())
                .eq("is_enable", Constant.IsEnable.ENABLE.getType()); //启用
        //1.构造数据
        List<QuestionsImportRequest> list = this.list(queryWrapper).stream().map(item -> {
            QuestionsImportRequest importRequest = new QuestionsImportRequest();
            BeanUtils.copyProperties(item, importRequest);
            return importRequest;
        }).collect(Collectors.toList());

        //2.加载模板流数据
        org.springframework.core.io.Resource resource = new ClassPathResource("excel-template/试题导出模板.xlsx");
//        FileInputStream fis = new FileInputStream(resource.getFile());
        InputStream inputStream = resource.getInputStream();

        //3、通过工具类下载文件
        new ExcelExportUtil(QuestionsImportRequest.class, Constant.ROW_INDEX, Constant.STYLE_INDEX).
                export(response, inputStream, list, "试题表.xlsx");
    }

    @Override
    public String getEvaluationRemainingTime(Integer userId) {
        //时长转成秒来计算
        //先查询该用户有没有开始考试，既redis中有没有跟该用户id相关联的key
        //如果没有，表示是开始测评，向redis中插入一条数据，key是前缀加用户id，value是当前时间，过期时间是测评时长加三十秒
        //如果有，则得到value，既开始测评的时间，用当前时间和开始测评时间做差，测评时长减去该差值
        //TODO 提交测评时删除redis中的该条数据
        //得到总时长，单位：分
        EvaluationRulesVO evaluationRules = evaluationRulesService.getEvaluationRules();
        Integer duration = evaluationRules.getDuration();
        ValueOperations<String, String> ops = stringRedisTemplate.opsForValue();
        String key = REMAINING_TINE_KEY + userId;
        String startTime = ops.get(key);
        // 剩余时间，单位：秒
        int remainingTime = duration * 60;
        if (StringUtils.isEmpty(startTime)) {
            //如果没有，表示是开始测评，向redis中插入一条数据，key是前缀加用户id，value是当前时间，过期时间是测评时长加十秒
            ops.set(key, String.valueOf(System.currentTimeMillis()), duration * 60 + 10, TimeUnit.SECONDS);
        } else {
            // 当前时间与开始时间得时间差，单位：毫秒
            long difference = System.currentTimeMillis() - Long.parseLong(startTime);
            remainingTime = (duration * 60) - (new Long(difference).intValue() / 1000);
            if (remainingTime < 0) {
                ExceptionCast.cast(CommonCode.EVALUATION_TIME_INVALID);
            }
        }
        return DateUtil.secondToTime(remainingTime);
    }

    @Override
    public void exportFailureRecord(HttpServletResponse response, String token) throws Exception {
        if (StringUtils.isEmpty(token)) {
            return;
        }
        ValueOperations<String, String> ops = stringRedisTemplate.opsForValue();
        //获取数据
        String record = ops.get(token);
        if (StringUtils.isEmpty(record)) {
            return;
        }


        List<QuestionsImportFailureVO> parse = JSON.parseObject(record, new TypeReference<List<QuestionsImportFailureVO>>() {
        });

        parse.sort(Comparator.comparing(QuestionsImportFailureVO::getIndex));

        //2.加载模板流数据
        org.springframework.core.io.Resource resource = new ClassPathResource("excel-template/试题导入失败数据导出模板.xlsx");
//        FileInputStream fis = new FileInputStream(resource.getFile());
        InputStream inputStream = resource.getInputStream();
        //3、通过工具类下载文件
        new ExcelExportUtil(QuestionsImportFailureVO.class, Constant.ROW_INDEX, Constant.STYLE_INDEX).
                export(response, inputStream, parse, "导入失败数据表.xlsx");
    }

//    @Override
//    public EvaluationRulesEntity queryNumberOfQuestions() {
//        EvaluationRulesEntity questions = this.getBaseMapper().queryNumberOfQuestions();
//        return questions;
//    }
//
////    测评试题是否禁用试题
//    @Override
//    public EvaluationRulesEntity queryNumberOfTatolQuestions() {
//        EvaluationRulesEntity evaluationRulesEntity = this.getBaseMapper().queryNumberOfTatolQuestions();
//        return evaluationRulesEntity;
//    }

}