package com.msdw.tms.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.msdw.tms.common.exception.ExceptionCast;
import com.msdw.tms.common.utils.*;
import com.msdw.tms.common.utils.poi.ExcelExportUtil;
import com.msdw.tms.dao.StudentDao;
import com.msdw.tms.dao.UserInfoDao;
import com.msdw.tms.entity.StudentEntity;
import com.msdw.tms.entity.UserInfoEntity;
import com.msdw.tms.entity.XlsxTemplateEntity;
import com.msdw.tms.entity.response.CommonCode;
import com.msdw.tms.entity.vo.QuestionsImportFailureVO;
import com.msdw.tms.entity.vo.StaffVo;
import com.msdw.tms.entity.vo.StudentImportFailureVo;
import com.msdw.tms.entity.vo.StudentVo;
import com.msdw.tms.service.AliyunOssService;
import com.msdw.tms.service.StudentService;
import com.msdw.tms.service.XlsxTemplateService;
import org.apache.commons.lang.StringUtils;
import org.apache.poi.ss.formula.functions.T;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ClassPathResource;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.util.*;
import java.util.concurrent.TimeUnit;

@Service("studentService")
public class StudentServiceImpl extends ServiceImpl<StudentDao, StudentEntity> implements StudentService {

    @Autowired
    private StudentDao studentDao;

    @Resource
    XlsxTemplateService xlsxTemplateService;

    @Resource
    AliyunOssService ossService;

    @Autowired
    private UserInfoDao userInfoDao;

    @Resource
    StringRedisTemplate stringRedisTemplate;

    @Override
    public Integer queryStudentIdNumber(String workNumber, Integer schoolId) {
        List<Integer> result = studentDao.queryStudentIdNumber(workNumber, schoolId);
        if (result.size() == 1) {
            return 1;
        } else {
            return 0;
        }
    }

    @Transactional
    @Override
    public boolean saveStudent(StudentVo studentvo) {
        studentvo.setIsdel(Constant.IsDel.NOT_DEL.getType());
        boolean save = studentDao.saveStudent(studentvo);
        return save;
    }

    @Override
    public PageUtils queryStudent(Integer schoolId, String searchContent, Integer page, Integer size) {
        Page<T> tPage = new Page<>(page, size);
        IPage<StudentVo> saveStudent = studentDao.queryStudent(tPage, searchContent, schoolId);
        PageUtils pageUtils = new PageUtils(saveStudent);
        return pageUtils;
    }

    @Override
    public R queryStudentDetails(Integer userId) {
        StudentVo saveStudent = studentDao.queryStudentDetails(userId);
        if (saveStudent != null) {
            return R.ok().put("data", saveStudent);
        } else {
            return R.error(300, "用户不存在！");
        }
    }

    @Transactional
    @Override
    public boolean deleteStudent(List<Integer> studentId) {

        try {
            Integer delete = studentDao.deleteStudent(studentId);
            return true;
        } catch (RuntimeException e) {
            e.printStackTrace();
            return false;
        }
    }

    @Override
    public boolean updateStudent(StudentVo studentVo) {
        try {
            Integer update = studentDao.updateStudent(studentVo);
            return true;
        } catch (RuntimeException e) {
            e.printStackTrace();
            return false;
        }
    }

    @Override
    public void updateWorkNumber(Integer studentId) {
        studentDao.updateWorkNumber("00", studentId);//将学号重置为00
    }

    @Override
    public R addStudentInfo(StudentVo studentVo) {
        R r = new R();
        boolean result = studentDao.addStudentInfo(studentVo);
        if (result == false) {
            throw new RuntimeException();//添加失败，抛出运行时异常让外层事务回滚
        } else {
            return R.ok("注册成功！");
        }

    }

    @Override
    public void downloadFiles(HttpServletResponse response) throws IOException {
        XlsxTemplateEntity xlsxTemplate = xlsxTemplateService.getById(ConstantUtils.XLSX_TEMPLATE_ID);
        ossService.downloadFiles(response, xlsxTemplate.getFileName());
    }

    @Transactional
    @Override
    public Map<String, String> upload(MultipartFile file, Integer schoolId) throws IOException {

        List<StudentVo> students = ExcelImportHelper.readStudent(file);
        if (students.size() <= 0) {
            ExceptionCast.cast(CommonCode.EXCEL_FILE_FORMAT_ERROR);
        }

        Map<String, String> errorMap = new HashMap<>();

        List<StudentImportFailureVo> failVo1 = new ArrayList<>();
        // 参数合法性校验，只能上传.xlsx后缀的文件
        if (StringUtils.isBlank(file.getOriginalFilename())
                || !file.getOriginalFilename().substring(file.getOriginalFilename().lastIndexOf(".")).equals(Constant.EXCEL_SUFFIX)) {
            ExceptionCast.cast(CommonCode.EXCEL_FILE_INVALID);
        }
        int size = students.size();
        Map<String, String> map = new HashMap<>();
        Long ii = 1L;//用于记录序列号
        try {
            for (int i = 0; i < students.size(); i++) {
                StudentVo student = students.get(i);

                String userName = student.getUserName();


                if (userName.equals("")) {
                    log.error("*姓名不能为空");
                    StudentImportFailureVo vo = new StudentImportFailureVo();
                    ++ii;
                    vo.setIndex(ii)
                            .setUserName(student.getUserName() + " 必填项:((姓名不能为空)")
                            .setAccount(student.getAccount())
                            .setWorkNumber(student.getWorkNumber())
                            .setPhone(student.getPhone()).setEmail(student.getEmail())
                    ;
                    failVo1.add(vo);
                    students.remove(i);
                    i--;
                    continue;
                }


                String phone = student.getPhone();
                String workNumber = student.getWorkNumber();

                if (workNumber.equals("")) {
                    log.error("*学号不能为空");
                    StudentImportFailureVo vo = new StudentImportFailureVo();
                    ++ii;
                    vo.setIndex(ii)
                            .setUserName(student.getUserName())
                            .setAccount(student.getAccount())
                            .setWorkNumber(student.getWorkNumber() + " 必填项:(*学号不能为空)")
                            .setPhone(student.getPhone()).setEmail(student.getEmail())
                    ;
                    failVo1.add(vo);
                    students.remove(i);
                    i--;
                    continue;
                }

                String account = student.getAccount();
                if (account.equals("")) {
                    log.error("*账号不能为空");
                    StudentImportFailureVo vo = new StudentImportFailureVo();
                    ++ii;
                    vo.setIndex(ii)
                            .setUserName(student.getUserName())
                            .setAccount(student.getAccount() + " 必填项:(*账号不能为空)")
                            .setWorkNumber(student.getWorkNumber())
                            .setPhone(student.getPhone()).setEmail(student.getEmail())
                    ;
                    failVo1.add(vo);
                    students.remove(i);
                    i--;
                    continue;
                }


                StudentVo studentVo = new StudentVo();
                //String schoolAppellationName = student.getSchoolAppellationName();
                List<Integer> result1 = studentDao.queryStudentIdNumber(workNumber, schoolId);
                List<StudentEntity> result4 = userInfoDao.queryStudentAccount(account);
                String email = student.getEmail();
                if (phone != null && phone != "") {
                    List<StudentEntity> result2 = userInfoDao.queryStudentPhone(phone);
                    if (result2.size() >= 1) {
                        log.error("该号码已被使用");
                        StudentImportFailureVo vo = new StudentImportFailureVo();
                        ++ii;
                        vo.setIndex(ii)
                                .setUserName(student.getUserName())
                                .setAccount(student.getAccount())
                                .setWorkNumber(student.getWorkNumber())
                                .setPhone(student.getPhone() + " (重复的号码)")
                                .setEmail(student.getEmail())
                        ;
                        failVo1.add(vo);
                        students.remove(i);
                        i--;
                        continue;
                    }
                    studentVo.setPhone(phone);
                }
                if (email != null && email != "") {
                    studentVo.setEmail(email);
                }
                // List<Integer> result3 = studentDao.querySchoolName(schoolAppellationName);
                // 唯一标示性账号
                studentVo.setUniqueIdentificationAccount(String.valueOf(System.currentTimeMillis())).
                        setPassword(ConstantUtils.INITIAL_PASSWORD).setToken(new ConstantUtils().token)
                        .setSchoolId(ConstantUtils.Keda_schoolId).setRoleId(ConstantUtils.STUDENT_ROLE)
                ;
                studentVo.setAccount(student.getAccount())
                        .setUserName(student.getUserName())
                        .setIsdel(Constant.IsDel.NOT_DEL.getType());


                if (result1.size() >= 1) {
                    log.error("该学号已存在");
                    StudentImportFailureVo vo = new StudentImportFailureVo();
                    ++ii;
                    vo.setIndex(ii)
                            .setUserName(student.getUserName())
                            .setAccount(student.getAccount())
                            .setWorkNumber(student.getWorkNumber() + " (重复的学号)")
                            .setPhone(student.getPhone()).setEmail(student.getEmail())
                    ;
//                            .setFailureMsg("重复的学号");
                    failVo1.add(vo);
//                    map.put("学号已存在序列："+System.currentTimeMillis(),"学号："+workNumber);
                    students.remove(i);
                    i--;
                    continue;
                } else if (result4.size() >= 1) {
                    log.error("账号已存在");
                    StudentImportFailureVo vo = new StudentImportFailureVo();
                    ++ii;
                    vo.setIndex(ii)
                            .setUserName(student.getUserName())
                            .setAccount(student.getAccount() + " (重复的账号)")
                            .setWorkNumber(student.getWorkNumber())
                            .setPhone(student.getPhone())
                            .setEmail(student.getEmail())
                    ;
//                            .setFailureMsg("重复的账号");
                    failVo1.add(vo);
//                    map.put("学号已存在序列："+System.currentTimeMillis(),"学号："+workNumber);
                    students.remove(i);
                    i--;
                    continue;
                }
                /*else if(result3.size()!=1){
                    log.error("不存在该院校");
                    StudentImportFailureVo vo = new StudentImportFailureVo();
                    ++ii;
                    vo.setIndex(ii).setUserName(student.getUserName()).setSchoolAppellationName(student.getSchoolAppellationName())
                            .setAccount(student.getAccount()).setWorkNumber(student.getWorkNumber()).setPhone(student.getPhone()).setEmail(student.getEmail())
                            .setFailureMsg("不存在的院校");
                    failVo1.add(vo);
//                    map.put("错误的学校名称序列："+System.currentTimeMillis(),"学校名称："+schoolAppellationName);
                    students.remove(i);
                    i--;
                    continue;
                }*/
                ++ii;
                if (!students.isEmpty() && students.size() > 0) {
                    studentVo.setSchoolId(schoolId);
                    userInfoDao.batchSaveUserInfo(studentVo);
                    student.setUserId(studentVo.getUserId()).setIsdel(Constant.IsDel.NOT_DEL.getType()).setPlatformId(ConstantUtils.PLATFORMID);
                    boolean b = studentDao.saveStudent(student);
                    if (!b) {
                        throw new RuntimeException();
                    }
                }


            }
        } catch (RuntimeException e) {
            log.error(e.getMessage());
            map.put("failura", "添加失败");
            throw new RuntimeException();
        }
        String token = "";

        // 有导入失败的数据，才会存入redis
        if (failVo1.size() > 0) {
            //生成token
            token = "FAILURE_IMPORT" + UUID.randomUUID().toString().replace("-", "");

            ValueOperations<String, String> ops = stringRedisTemplate.opsForValue();
            String failureVOJson = JSON.toJSONString(failVo1);

            ops.set(token, failureVOJson, 30 * 60, TimeUnit.SECONDS);
        }
        map.put("token", token);
        map.put("successNum", (size - failVo1.size()) + "");
        map.put("failureNum", failVo1.size() + "");

        return map;
    }

    @Override
    public void exportFailureRecord(HttpServletResponse response, String token) throws Exception {
        if (StringUtils.isEmpty(token)) {
            return;
        }
        ValueOperations<String, String> ops = stringRedisTemplate.opsForValue();
        //获取数据
        String record = ops.get(token);
        if (StringUtils.isEmpty(record)) {
            return;
        }
        List<StudentImportFailureVo> parse = JSON.parseObject(record, new TypeReference<List<StudentImportFailureVo>>() {
        });

        parse.sort(Comparator.comparing(StudentImportFailureVo::getIndex));

        //2.加载模板流数据
        org.springframework.core.io.Resource resource = new ClassPathResource("excel-template/学生导入失败数据导出模板(1).xlsx");
        InputStream inputStream = resource.getInputStream();

        //3、通过工具类下载文件
        new ExcelExportUtil(StudentImportFailureVo.class, Constant.ROW_INDEX, Constant.STYLE_INDEX).
                export(response, inputStream, parse, "学生信息导入失败表(1).xlsx");

    }

    @Override
    public R queryAccount(String account, Integer schoolId) {
        List<StudentVo> result = studentDao.queryAccount(account, schoolId);
        List<StaffVo> vos = userInfoDao.queryAccount(account);
        if (result.size() > 1) {
            return R.error("系统存在多个相同的账号：" + account);
        } else if (result.size() == 1) {
            return R.ok().put("data", result);
        } else if (vos.size() > 0) {
            return R.error("账号已存在");
        }
        return R.ok();

    }

    @Override
    public int queryStudentByWorkNumber(StudentVo studentVo) {
        return studentDao.queryStudentByWorkNumber(studentVo);
    }

}
