package com.msdw.tms.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.msdw.tms.common.utils.PageUtils;
import com.msdw.tms.dao.SysPermissionMapper;
import com.msdw.tms.dao.SysRoleMapper;
import com.msdw.tms.dao.SysRolePermissionMapper;
import com.msdw.tms.dao.SysUserRoleMapper;
import com.msdw.tms.entity.SysPermission;
import com.msdw.tms.entity.SysRole;
import com.msdw.tms.entity.SysRolePermission;
import com.msdw.tms.entity.request.RolePermissionRequest;
import com.msdw.tms.entity.response.RolePermissionResponse;
import com.msdw.tms.service.ISysPermissionService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 菜单权限表 服务实现类
 * </p>
 *
 * @author Qyq
 * @since 2021-02-24
 */
@Service
public class SysPermissionServiceImpl extends ServiceImpl<SysPermissionMapper, SysPermission> implements ISysPermissionService {

    @Resource
    private SysRolePermissionMapper rolePermissionMapper;

    @Resource
    private SysRoleMapper roleMapper;

    @Resource
    private SysUserRoleMapper userRoleMapper;

    @Override
    public List<SysPermission> listWithTree() {
/*        List<SysPermission>list = this.getBaseMapper().queryPermissionTree();
        return getPermissionTreeList(list,0L);*/
        System.out.println("this.list()=================="+this.list());
        return getPermissionTreeList(this.list(), 0L);
    }

    private List<SysPermission> getPermissionTreeList(List<SysPermission> list, Long parentId){
        return list.stream()
                .filter(item -> parentId.equals(item.getParentId()))
                .peek(item -> item.setChildren(getPermissionTreeList(list, item.getId())))
                .sorted(Comparator.comparingInt(menu -> (menu.getSort() == null ? 0 : menu.getSort())))
                .collect(Collectors.toList());
    }

    @Override
    public int uniqueRoleName(String name) {
        return rolePermissionMapper.uniqueRoleName(name);
    }

    @Override
    public int addHrmsRole(RolePermissionRequest rolePermission) {
        int request = roleMapper.addHrmsRole(rolePermission);
        return request;
    }

    @Override
    public boolean addHrmsRolePermission(List<RolePermissionRequest> rolePermissionList) {
        return rolePermissionMapper.addHrmsRolePermission(rolePermissionList);
    }

    @Override
    public int updateHrmsRole(RolePermissionRequest rolePermission) {
        return roleMapper.updateHrmsRole(rolePermission);
    }

    @Override
    public boolean deleteHrmsRolePermission(Long roleId) {
        return rolePermissionMapper.deleteHrmsRolePermissionSingle(roleId);
    }

    @Override
    public boolean deleteHrmsRolePermission(Long[] roleIds) {
        return rolePermissionMapper.deleteHrmsRolePermission(Arrays.asList(roleIds));
    }

    @Override
    public int queryCountByRoleId(Long[] roleIds) {
        int count = userRoleMapper.queryCountByRoleId(Arrays.asList(roleIds));
        return count;
    }

    @Override
    public boolean deleteHrmsRoleByRoleId(Long[] roleIds) {
        return roleMapper.deleteHrmsRoleByRoleId(Arrays.asList(roleIds));
    }

    @Override
    public List<SysRolePermission> queryPermissionArrById(Long roleId) {
        return rolePermissionMapper.queryPermissionArrById(roleId);
    }

    @Override
    public SysRole queryRoleMsg(Long roleId) {
        return roleMapper.selectById(roleId);
    }

    @Override
    public PageUtils queryPage(Integer page, Integer size, String name) {
        //分页参数
        if (page <= 0) {
            page = 1;
        }
        if (size <= 0) {
            size = 10;
        }
        IPage<RolePermissionResponse> systemLogVo = this.getBaseMapper().pageByCondition(new Page(page, size),name);

        return new PageUtils(systemLogVo);
    }
}
