package com.msdw.tms.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.msdw.tms.common.utils.*;
import com.msdw.tms.config.CheckCodeUtils;
import com.msdw.tms.config.EmailUtils;
import com.msdw.tms.config.SendSMS;
import com.msdw.tms.dao.UserDao;
import com.msdw.tms.dao.UserInfoDao;
import com.msdw.tms.entity.StudentEntity;
import com.msdw.tms.entity.UserInfoEntity;
import com.msdw.tms.entity.vo.CheckVo;
import com.msdw.tms.entity.vo.StudentVo;
import com.msdw.tms.entity.vo.UserProsonalEntityVo;
import com.msdw.tms.service.UserInfoService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

@Slf4j
@Service("userInfoService")
public class UserInfoServiceImpl extends ServiceImpl<UserInfoDao, UserInfoEntity> implements UserInfoService {
    @Autowired
    private UserInfoDao userInfoDao;

    @Autowired
    private UserDao userDao;

    @Autowired
    private StringRedisTemplate stringRedisTemplate;


    @Override
    public PageUtils queryPage(Map<String, Object> params) {

        IPage<UserInfoEntity> page = this.page(
                new Query<UserInfoEntity>().getPage(params),
                new QueryWrapper<UserInfoEntity>()
        );
        return new PageUtils(page);
    }


    @Override
    public int add(UserInfoEntity userInfoEntity) {
        UserInfoDao userDao = this.getBaseMapper();
        userInfoEntity.setToken(new ConstantUtils().token);
        int userId = userDao.add(userInfoEntity);
        return userId;


    }
    //    @Override
//public R add(UserInfoEntity userInfoEntity) {
//    R r = new R();
//    UserInfoDao userDao = this.getBaseMapper();
//    Boolean vo = userDao.add(userInfoEntity);
//    if (vo==false){
//        r.put("code",300);
//        r.put("errmessage","注册失败！");
//        return r;
//    }else{
//        r.put("code",200);
//        r.put("errmessage","注册成功！");
//        return r;
//    }
//}
    @Override
    public void updateUserInfoById(UserInfoEntity entity){
        userInfoDao.userInfupdateUserInfoById(entity);
    }

    @Override
    public String queryPasword(Integer userId){
        String s = userInfoDao.queryUserPassword(userId);
        return s;
    }

    @Transactional
    @Override
    public int saveUserInfo(UserInfoEntity userInfo) {
        userInfo.setUniqueIdentificationAccount(String.valueOf(System.currentTimeMillis()))
                .setIsdel(Constant.IsDel.NOT_DEL.getType()).setPassword(ConstantUtils.INITIAL_PASSWORD);
        int userId = userInfoDao.saveUserInfo(userInfo);
        return userId;
    }

    @Transactional
    @Override
    public Integer initialPassword(Integer userId) {

        Integer initial = userInfoDao.initialPassword(userId, ConstantUtils.INITIAL_PASSWORD);
        return initial;
    }

    @Transactional
    @Override
    public void disableAccount(StudentVo studentIds) {
        userInfoDao.disableAccount(studentIds);
    }

    @Override
    public Integer queryStudentAccount(String account,Integer schoolId) {
        List<StudentEntity> result = userInfoDao.queryStudentAccount(account);
        if (result.size()==1){
            return 1;
        }else{
            return 0;
        }
    }

    @Override
    public Integer queryStudentPhone(String phone) {
        List<StudentEntity> result = userInfoDao.queryStudentPhone(phone);
        if (result.size()==1){
            return 1;
        }else{
            return 0;
        }
    }

    @Override
    public HashMap<String, UserProsonalEntityVo> queryUserInfo(Integer id) {
        HashMap map = new HashMap();
        //用户sql user
        List<UserProsonalEntityVo> userInfo = userInfoDao.queryPersonalInfo(id);
        map.put("userInfo",userInfo);

        List<UserProsonalEntityVo> prosonalEntityESVos = userInfoDao.queryPersonalFileES(id);//档案里无学校信息时
        map.put("userPersonalFile",prosonalEntityESVos);

        return map;
    }

    @Override
    public void updateAccount( Integer userId) {
        userInfoDao.setNUM(userId);
    }

    @Override
    public R checkCode(CheckVo checkVo) {

        ValueOperations<String, String> ops = stringRedisTemplate.opsForValue();
        String opener = ops.get(checkVo.getOpener());
        if (opener==null){
            return R.error("验证码已过期或不存在,请重新发送验证码");
        }else if (opener.equals(checkVo.getCode())){
            return R.ok();
        }else {
            return R.error("failure");
        }

    }

    @Override
    @Transactional
    public R resetPassword(CheckVo checkVo) {
        userInfoDao.resetPassword(checkVo);
        return R.ok();
    }

    @Override
    public R findPasswordByPhone(String phone) {
        List<Integer> list = userInfoDao.queryPhone(phone);
        switch (list.size()){
            case 0:return R.error("该手机号未绑定账号！！！");
            case 1:
                String code = new CheckCodeUtils().sendCode();
                String opener = UUID.randomUUID().toString().replace("-", "");
                SendSMS.sendCode(phone,code);
                ValueOperations<String, String> ops = stringRedisTemplate.opsForValue();
                ops.set(opener, code, 5*60, TimeUnit.SECONDS);//5分钟
                CheckVo checkVo = new CheckVo();
                checkVo.setUserId(list.get(0)).setOpener(opener);
                return R.ok().put("data",checkVo);
            default:
                log.info("该手机号存在绑定多个账号！！！");
                return R.error("该手机号号绑定异常，请联系管理员！");
        }
    }

    @Override
    public R findPasswordByEmail(String email) {
        List<Integer> list = userInfoDao.queryEmail(email);
        switch (list.size()){
            case 0:return R.error("该邮箱未绑定账号！！！");
            case 1:
                String code = new CheckCodeUtils().sendCode();
                String opener = UUID.randomUUID().toString().replace("-", "");
                EmailUtils.sendEmail(email,code);
                ValueOperations<String, String> ops = stringRedisTemplate.opsForValue();
                ops.set(opener, code, 5*60, TimeUnit.SECONDS);//5分钟
                CheckVo checkVo = new CheckVo();
                checkVo.setUserId(list.get(0)).setOpener(opener);
                return R.ok().put("data",checkVo);
            default:
                log.info("该邮箱存在绑定多个账号！！！");
                return R.error("该邮箱号绑定异常，请联系管理员！");

        }
    }

    @Override
    public R queryAccount() {
        userInfoDao.updateUserDetail();//将已被删除的账号恢复
        UserInfoEntity result = userInfoDao.queryUserAccount();//取一个账号
        return R.ok().put("data",result);
    }

    @Override
    public R getUserId(String token) {
        Integer userId = userInfoDao.getUserId(token);
        return R.ok().put("userId", userId);
    }

    @Override
    public Integer queryAccountExist(String account,Integer userId) {
        return userInfoDao.queryAccountExist(account,userId);
    }
}
