package com.msdw.tms.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.msdw.tms.common.utils.Constant;
import com.msdw.tms.common.utils.PageUtils;
import com.msdw.tms.common.utils.Query;
import com.msdw.tms.common.utils.R;
import com.msdw.tms.config.CheckCodeUtils;
import com.msdw.tms.config.EmailUtils;
import com.msdw.tms.config.SendSMS;
import com.msdw.tms.dao.UserDao;
import com.msdw.tms.entity.UserEntity;
import com.msdw.tms.entity.UserPersonalFileEntity;
import com.msdw.tms.entity.vo.CheckVo;
import com.msdw.tms.entity.vo.UserEntityVo;
import com.msdw.tms.entity.vo.UserProsonalEntityVo;
import com.msdw.tms.service.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.concurrent.TimeUnit;

@Service("userService")
public class UserServiceImpl extends ServiceImpl<UserDao, UserEntity> implements UserService {
    @Autowired
    private UserDao userMapper;

    @Autowired
    private StringRedisTemplate stringRedisTemplate;


    @Override
    public PageUtils queryPage(Map<String, Object> params) {

        IPage<UserEntity> page = this.page(
                new Query<UserEntity>().getPage(params),
                new QueryWrapper<UserEntity>()
        );
        return new PageUtils(page);
    }

    @Override
    public UserEntityVo queryUserId(Integer userId) {

        UserDao userDao = this.getBaseMapper();
        UserEntity userEntity = new UserEntity();
        userEntity.setUserid(userId).setIsdel(Constant.IsDel.NOT_DEL.getType());
        UserEntityVo vo = userDao.getByUserId(userEntity);
        return vo;
    }


    @Override
    public void updateUserPersonal(UserPersonalFileEntity personalFile) {
        userMapper.updateUserPersonal(personalFile);
    }

    @Override
    public HashMap<String,UserProsonalEntityVo> queryUserInfo(Integer id) {
        HashMap map = new HashMap();
        //用户sql user
        List<UserProsonalEntityVo> userInfo = userMapper.queryUserInfo(id);
        map.put("userInfo",userInfo);

        List<UserProsonalEntityVo> prosonalEntityESVos = userMapper.queryPersonalFileES(id);//档案里无学校信息时
        map.put("userPersonalFile",prosonalEntityESVos);

        return map;
    }

    @Override
    public void insertPersonalInfo(UserPersonalFileEntity personalFile) {
        userMapper.insertPersonalInfo(personalFile);
    }

    @Override
    public R bindEmail(CheckVo checkVo) {

        ValueOperations<String, String> ops = stringRedisTemplate.opsForValue();
        String opener = ops.get(checkVo.getOpener());//获取验证码

        if (opener==null){
            return R.error("验证码已过期或不存在,请重新发送验证码");
        } else if (opener.equals(checkVo.getCode())){
            boolean bind = userMapper.bindEmail(checkVo);
            if (bind){
                return R.ok();
            }else {
                return R.error("更新失败");
            }
        }
        return R.error("验证失败");
    }

    @Transactional
    @Override
    public List<String> findEmailCode(String email,Integer types) {
        List<String> bind = userMapper.findEmailCode(email,types);
        return bind;
    }
    @Transactional
    @Override
    public List<String> findEmailSetTime(String email,String checkCode,Integer types) {
        List<String> bind = userMapper.findEmailSetTime(email,checkCode,types);
        return bind;
    }

    @Override
    public boolean queryEmial(CheckVo checkVo){
        List<Integer> re = userMapper.queryEmial(checkVo);
        if (re.size()>0){
            return true;
        }else{
            return false;
        }
    }
    @Override
    public boolean queryPhone(CheckVo checkVo){
        List<Integer> re = userMapper.queryPhone(checkVo);
        if (re.size()>0){
            return true;
        }else{
            return false;
        }
    }
    @Override
    public R saveCheckCode(CheckVo checkVo) {
        CheckCodeUtils checkCodeUtils = new CheckCodeUtils();
        //获取六位随机数验证码
        String code = checkCodeUtils.sendCode();
        EmailUtils.sendEmail(checkVo.getEmail(),code);//邮箱验证码,发送至新邮箱

        String opener = UUID.randomUUID().toString().replace("-", "");
        ValueOperations<String, String> ops = stringRedisTemplate.opsForValue();
        ops.set(opener, code, 5*60, TimeUnit.SECONDS);//5分钟

        return R.ok().put("data",opener);
    }

    @Override
    public R sendCheckPhoneCode(CheckVo checkVo) {
        CheckCodeUtils checkCodeUtils = new CheckCodeUtils();
        //获取六位随机数验证码
        String code = checkCodeUtils.sendCode();
        SendSMS.sendCode(checkVo.getPhone(),code);//发送邮箱验证码

        String opener = UUID.randomUUID().toString().replace("-", "");
        ValueOperations<String, String> ops = stringRedisTemplate.opsForValue();
        ops.set(opener, code, 5*60, TimeUnit.SECONDS);//5分钟

        return R.ok().put("data",opener);

    }

    @Transactional
    @Override
    public R bindPhone(CheckVo checkVo) {
        ValueOperations<String, String> ops = stringRedisTemplate.opsForValue();
        String opener = ops.get(checkVo.getOpener());//获取验证码
        if (opener==null){
            return R.error("验证码已过期或不存在,请重新发送验证码");
        } else if (opener.equals(checkVo.getCode())){
            boolean bind = userMapper.bindPhone(checkVo);
            if (bind){
                return R.ok();
            }else {
                return R.error("更新失败");
            }

        }
        return R.error("验证失败");

    }
    @Transactional
    @Override
    public List<String> findPhoneCode(String phone,Integer types) {
        List<String> bind = userMapper.findPhoneCode(phone,types);
        return bind;
    }
    @Transactional
    @Override
    public List<String> findPhoneSetTime(String phone,String checkCode,Integer types) {
        List<String> bind = userMapper.findPhoneSetTime(phone,checkCode,types);
        return bind;
    }
}
