package com.msdw.tms.service;

import com.msdw.tms.common.utils.FilesResult;
import com.msdw.tms.common.utils.PageUtils;
import com.msdw.tms.entity.request.QuestionsAddRequest;
import com.msdw.tms.entity.request.QuestionsQueryRequest;
import com.msdw.tms.entity.request.QuestionsUpdateRequest;
import com.msdw.tms.entity.vo.EvaluationVO;
import com.msdw.tms.entity.vo.QuestionsDetailVO;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.disk.DiskFileItem;
import org.apache.commons.io.IOUtils;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.nio.file.Files;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 记录试题信息
 *
 * @author gongsj
 */
@SpringBootTest
class QuestionsServiceTest {

    @Autowired
    QuestionsService questionsService;

    @Test
    void queryQuestionsPage() {
        Integer page = 1;
        Integer size = 10;
        QuestionsQueryRequest request = new QuestionsQueryRequest();
        PageUtils pageUtils = questionsService.queryQuestionsPage(page, size, request);
        System.out.println(pageUtils.toString());
    }

    @Test
    void getQuestionDetailById() {
        Integer id = 1;
        QuestionsDetailVO detail = questionsService.getQuestionDetailById(id);
        System.out.println(detail.toString());
    }

    @Test
    @Transactional
    void saveQuestion() {
        QuestionsAddRequest questions = new QuestionsAddRequest();
        System.out.println(questionsService.saveQuestion(questions));
    }

    @Test
    @Transactional
    void updateQuestionById() {
        QuestionsUpdateRequest questions = new QuestionsUpdateRequest();
        System.out.println(questionsService.updateQuestionById(questions));
    }

//    @JustTest
//    @Transactional
//    void isnable() {
//        Integer id = 1;
//        System.out.println(questionsService.isEnable(id));
//    }

    @Test
    @Transactional
    void deleteByIds() {
        List<Integer> asList = new ArrayList<>();
        System.out.println(questionsService.deleteByIds(asList));
    }

    @Test
    @Transactional
    void importQuestion() throws IOException {
        File file = new File("D:\\pic\\1.jpg");
        FileItem fileItem = new DiskFileItem("file",
                Files.probeContentType(file.toPath()),
                false,
                file.getName(),
                (int) file.length(),
                file.getParentFile());
        IOUtils.copy(new FileInputStream(file), fileItem.getOutputStream());
        MultipartFile multipartFile = new CommonsMultipartFile(fileItem);

        Map<String, String> map = questionsService.importQuestion(multipartFile);

        System.out.println(map.toString());
    }

    @Test
    void uploadFiles() throws IOException {
        File file = new File("D:\\pic\\1.jpg");
        FileItem fileItem = new DiskFileItem("file",
                Files.probeContentType(file.toPath()),
                false,
                file.getName(),
                (int) file.length(),
                file.getParentFile());
        IOUtils.copy(new FileInputStream(file), fileItem.getOutputStream());
        MultipartFile multipartFile = new CommonsMultipartFile(fileItem);
        FilesResult filesResult = questionsService.uploadFiles(multipartFile);
        System.out.println(filesResult.toString());
    }

    @Test
    void downloadFiles() throws IOException {
        HttpServletResponse response = null;
        questionsService.downloadFiles(response);
    }

//    @JustTest
//    void evaluation() {
//        EvaluationVO evaluation = questionsService.evaluation();
//        System.out.println(evaluation.toString());
//    }


}

